# DO NOT EDIT! This file is automatically generated
import marshmallow

from commercetools import helpers, types
from commercetools._schemas._common import (
    BaseResourceSchema,
    ReferenceSchema,
    ResourceIdentifierSchema,
)

__all__ = [
    "SubRateSchema",
    "TaxCategoryAddTaxRateActionSchema",
    "TaxCategoryChangeNameActionSchema",
    "TaxCategoryDraftSchema",
    "TaxCategoryPagedQueryResponseSchema",
    "TaxCategoryReferenceSchema",
    "TaxCategoryRemoveTaxRateActionSchema",
    "TaxCategoryReplaceTaxRateActionSchema",
    "TaxCategoryResourceIdentifierSchema",
    "TaxCategorySchema",
    "TaxCategorySetDescriptionActionSchema",
    "TaxCategorySetKeyActionSchema",
    "TaxCategoryUpdateActionSchema",
    "TaxCategoryUpdateSchema",
    "TaxRateDraftSchema",
    "TaxRateSchema",
]


class SubRateSchema(marshmallow.Schema):
    """Marshmallow schema for :class:`commercetools.types.SubRate`."""

    name = marshmallow.fields.String(allow_none=True)
    amount = marshmallow.fields.Float(allow_none=True)

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        return types.SubRate(**data)


class TaxCategoryDraftSchema(marshmallow.Schema):
    """Marshmallow schema for :class:`commercetools.types.TaxCategoryDraft`."""

    name = marshmallow.fields.String(allow_none=True)
    description = marshmallow.fields.String(allow_none=True, missing=None)
    rates = helpers.LazyNestedField(
        nested="commercetools._schemas._tax_category.TaxRateDraftSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        many=True,
    )
    key = marshmallow.fields.String(allow_none=True, missing=None)

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        return types.TaxCategoryDraft(**data)


class TaxCategoryPagedQueryResponseSchema(marshmallow.Schema):
    """Marshmallow schema for :class:`commercetools.types.TaxCategoryPagedQueryResponse`."""

    limit = marshmallow.fields.Integer(allow_none=True)
    count = marshmallow.fields.Integer(allow_none=True)
    total = marshmallow.fields.Integer(allow_none=True, missing=None)
    offset = marshmallow.fields.Integer(allow_none=True)
    results = helpers.LazyNestedField(
        nested="commercetools._schemas._tax_category.TaxCategorySchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        many=True,
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        return types.TaxCategoryPagedQueryResponse(**data)


class TaxCategoryReferenceSchema(ReferenceSchema):
    """Marshmallow schema for :class:`commercetools.types.TaxCategoryReference`."""

    obj = helpers.LazyNestedField(
        nested="commercetools._schemas._tax_category.TaxCategorySchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["type_id"]
        return types.TaxCategoryReference(**data)


class TaxCategoryResourceIdentifierSchema(ResourceIdentifierSchema):
    """Marshmallow schema for :class:`commercetools.types.TaxCategoryResourceIdentifier`."""

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["type_id"]
        return types.TaxCategoryResourceIdentifier(**data)


class TaxCategorySchema(BaseResourceSchema):
    """Marshmallow schema for :class:`commercetools.types.TaxCategory`."""

    id = marshmallow.fields.String(allow_none=True)
    version = marshmallow.fields.Integer(allow_none=True)
    created_at = marshmallow.fields.DateTime(allow_none=True, data_key="createdAt")
    last_modified_at = marshmallow.fields.DateTime(
        allow_none=True, data_key="lastModifiedAt"
    )
    last_modified_by = helpers.LazyNestedField(
        nested="commercetools._schemas._common.LastModifiedBySchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
        data_key="lastModifiedBy",
    )
    created_by = helpers.LazyNestedField(
        nested="commercetools._schemas._common.CreatedBySchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
        data_key="createdBy",
    )
    name = marshmallow.fields.String(allow_none=True)
    description = marshmallow.fields.String(allow_none=True, missing=None)
    rates = helpers.LazyNestedField(
        nested="commercetools._schemas._tax_category.TaxRateSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        many=True,
    )
    key = marshmallow.fields.String(allow_none=True, missing=None)

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        return types.TaxCategory(**data)


class TaxCategoryUpdateActionSchema(marshmallow.Schema):
    """Marshmallow schema for :class:`commercetools.types.TaxCategoryUpdateAction`."""

    action = marshmallow.fields.String(allow_none=True)

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.TaxCategoryUpdateAction(**data)


class TaxCategoryUpdateSchema(marshmallow.Schema):
    """Marshmallow schema for :class:`commercetools.types.TaxCategoryUpdate`."""

    version = marshmallow.fields.Integer(allow_none=True)
    actions = marshmallow.fields.List(
        helpers.Discriminator(
            discriminator_field=("action", "action"),
            discriminator_schemas={
                "addTaxRate": "commercetools._schemas._tax_category.TaxCategoryAddTaxRateActionSchema",
                "changeName": "commercetools._schemas._tax_category.TaxCategoryChangeNameActionSchema",
                "removeTaxRate": "commercetools._schemas._tax_category.TaxCategoryRemoveTaxRateActionSchema",
                "replaceTaxRate": "commercetools._schemas._tax_category.TaxCategoryReplaceTaxRateActionSchema",
                "setDescription": "commercetools._schemas._tax_category.TaxCategorySetDescriptionActionSchema",
                "setKey": "commercetools._schemas._tax_category.TaxCategorySetKeyActionSchema",
            },
            unknown=marshmallow.EXCLUDE,
            allow_none=True,
        ),
        allow_none=True,
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        return types.TaxCategoryUpdate(**data)


class TaxRateDraftSchema(marshmallow.Schema):
    """Marshmallow schema for :class:`commercetools.types.TaxRateDraft`."""

    name = marshmallow.fields.String(allow_none=True)
    amount = marshmallow.fields.Float(allow_none=True, missing=None)
    included_in_price = marshmallow.fields.Bool(
        allow_none=True, data_key="includedInPrice"
    )
    country = marshmallow.fields.String()
    state = marshmallow.fields.String(allow_none=True, missing=None)
    sub_rates = helpers.LazyNestedField(
        nested="commercetools._schemas._tax_category.SubRateSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        many=True,
        missing=None,
        data_key="subRates",
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        return types.TaxRateDraft(**data)


class TaxRateSchema(marshmallow.Schema):
    """Marshmallow schema for :class:`commercetools.types.TaxRate`."""

    id = marshmallow.fields.String(allow_none=True, missing=None)
    name = marshmallow.fields.String(allow_none=True)
    amount = marshmallow.fields.Float(allow_none=True)
    included_in_price = marshmallow.fields.Bool(
        allow_none=True, data_key="includedInPrice"
    )
    country = marshmallow.fields.String()
    state = marshmallow.fields.String(allow_none=True, missing=None)
    sub_rates = helpers.LazyNestedField(
        nested="commercetools._schemas._tax_category.SubRateSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        many=True,
        missing=None,
        data_key="subRates",
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        return types.TaxRate(**data)


class TaxCategoryAddTaxRateActionSchema(TaxCategoryUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.TaxCategoryAddTaxRateAction`."""

    tax_rate = helpers.LazyNestedField(
        nested="commercetools._schemas._tax_category.TaxRateDraftSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        data_key="taxRate",
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.TaxCategoryAddTaxRateAction(**data)


class TaxCategoryChangeNameActionSchema(TaxCategoryUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.TaxCategoryChangeNameAction`."""

    name = marshmallow.fields.String(allow_none=True)

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.TaxCategoryChangeNameAction(**data)


class TaxCategoryRemoveTaxRateActionSchema(TaxCategoryUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.TaxCategoryRemoveTaxRateAction`."""

    tax_rate_id = marshmallow.fields.String(allow_none=True, data_key="taxRateId")

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.TaxCategoryRemoveTaxRateAction(**data)


class TaxCategoryReplaceTaxRateActionSchema(TaxCategoryUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.TaxCategoryReplaceTaxRateAction`."""

    tax_rate_id = marshmallow.fields.String(allow_none=True, data_key="taxRateId")
    tax_rate = helpers.LazyNestedField(
        nested="commercetools._schemas._tax_category.TaxRateDraftSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        data_key="taxRate",
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.TaxCategoryReplaceTaxRateAction(**data)


class TaxCategorySetDescriptionActionSchema(TaxCategoryUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.TaxCategorySetDescriptionAction`."""

    description = marshmallow.fields.String(allow_none=True, missing=None)

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.TaxCategorySetDescriptionAction(**data)


class TaxCategorySetKeyActionSchema(TaxCategoryUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.TaxCategorySetKeyAction`."""

    key = marshmallow.fields.String(allow_none=True, missing=None)

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.TaxCategorySetKeyAction(**data)



# DO NOT EDIT! This file is automatically generated
import datetime
import enum
import typing

from commercetools.types._abstract import _BaseType
from commercetools.types._common import (
    BaseResource,
    Reference,
    ReferenceTypeId,
    ResourceIdentifier,
)

if typing.TYPE_CHECKING:
    from ._common import Address, CreatedBy, LastModifiedBy
    from ._customer_group import CustomerGroupReference, CustomerGroupResourceIdentifier
    from ._store import StoreKeyReference, StoreResourceIdentifier
    from ._type import (
        CustomFields,
        CustomFieldsDraft,
        FieldContainer,
        TypeResourceIdentifier,
    )
__all__ = [
    "AnonymousCartSignInMode",
    "Customer",
    "CustomerAddAddressAction",
    "CustomerAddBillingAddressIdAction",
    "CustomerAddShippingAddressIdAction",
    "CustomerAddStoreAction",
    "CustomerChangeAddressAction",
    "CustomerChangeEmailAction",
    "CustomerChangePassword",
    "CustomerCreateEmailToken",
    "CustomerCreatePasswordResetToken",
    "CustomerDraft",
    "CustomerEmailVerify",
    "CustomerPagedQueryResponse",
    "CustomerReference",
    "CustomerRemoveAddressAction",
    "CustomerRemoveBillingAddressIdAction",
    "CustomerRemoveShippingAddressIdAction",
    "CustomerRemoveStoreAction",
    "CustomerResetPassword",
    "CustomerResourceIdentifier",
    "CustomerSetCompanyNameAction",
    "CustomerSetCustomFieldAction",
    "CustomerSetCustomTypeAction",
    "CustomerSetCustomerGroupAction",
    "CustomerSetCustomerNumberAction",
    "CustomerSetDateOfBirthAction",
    "CustomerSetDefaultBillingAddressAction",
    "CustomerSetDefaultShippingAddressAction",
    "CustomerSetExternalIdAction",
    "CustomerSetFirstNameAction",
    "CustomerSetKeyAction",
    "CustomerSetLastNameAction",
    "CustomerSetLocaleAction",
    "CustomerSetMiddleNameAction",
    "CustomerSetSalutationAction",
    "CustomerSetStoresAction",
    "CustomerSetTitleAction",
    "CustomerSetVatIdAction",
    "CustomerSignInResult",
    "CustomerSignin",
    "CustomerToken",
    "CustomerUpdate",
    "CustomerUpdateAction",
]


class AnonymousCartSignInMode(enum.Enum):
    MERGE_WITH_EXISTING_CUSTOMER_CART = "MergeWithExistingCustomerCart"
    USE_AS_NEW_ACTIVE_CUSTOMER_CART = "UseAsNewActiveCustomerCart"


class Customer(BaseResource):
    #: :class:`str`
    id: str
    #: :class:`int`
    version: int
    #: :class:`datetime.datetime` `(Named` ``createdAt`` `in Commercetools)`
    created_at: datetime.datetime
    #: :class:`datetime.datetime` `(Named` ``lastModifiedAt`` `in Commercetools)`
    last_modified_at: datetime.datetime
    #: Optional :class:`commercetools.types.LastModifiedBy` `(Named` ``lastModifiedBy`` `in Commercetools)`
    last_modified_by: typing.Optional["LastModifiedBy"]
    #: Optional :class:`commercetools.types.CreatedBy` `(Named` ``createdBy`` `in Commercetools)`
    created_by: typing.Optional["CreatedBy"]
    #: Optional :class:`str` `(Named` ``customerNumber`` `in Commercetools)`
    customer_number: typing.Optional[str]
    #: :class:`str`
    email: str
    #: :class:`str`
    password: str
    #: Optional :class:`str` `(Named` ``firstName`` `in Commercetools)`
    first_name: typing.Optional[str]
    #: Optional :class:`str` `(Named` ``lastName`` `in Commercetools)`
    last_name: typing.Optional[str]
    #: Optional :class:`str` `(Named` ``middleName`` `in Commercetools)`
    middle_name: typing.Optional[str]
    #: Optional :class:`str`
    title: typing.Optional[str]
    #: Optional :class:`datetime.date` `(Named` ``dateOfBirth`` `in Commercetools)`
    date_of_birth: typing.Optional[datetime.date]
    #: Optional :class:`str` `(Named` ``companyName`` `in Commercetools)`
    company_name: typing.Optional[str]
    #: Optional :class:`str` `(Named` ``vatId`` `in Commercetools)`
    vat_id: typing.Optional[str]
    #: List of :class:`commercetools.types.Address`
    addresses: typing.List["Address"]
    #: Optional :class:`str` `(Named` ``defaultShippingAddressId`` `in Commercetools)`
    default_shipping_address_id: typing.Optional[str]
    #: Optional list of :class:`str` `(Named` ``shippingAddressIds`` `in Commercetools)`
    shipping_address_ids: typing.Optional[typing.List[str]]
    #: Optional :class:`str` `(Named` ``defaultBillingAddressId`` `in Commercetools)`
    default_billing_address_id: typing.Optional[str]
    #: Optional list of :class:`str` `(Named` ``billingAddressIds`` `in Commercetools)`
    billing_address_ids: typing.Optional[typing.List[str]]
    #: :class:`bool` `(Named` ``isEmailVerified`` `in Commercetools)`
    is_email_verified: bool
    #: Optional :class:`str` `(Named` ``externalId`` `in Commercetools)`
    external_id: typing.Optional[str]
    #: Optional :class:`commercetools.types.CustomerGroupReference` `(Named` ``customerGroup`` `in Commercetools)`
    customer_group: typing.Optional["CustomerGroupReference"]
    #: Optional :class:`commercetools.types.CustomFields`
    custom: typing.Optional["CustomFields"]
    #: Optional :class:`str`
    locale: typing.Optional[str]
    #: Optional :class:`str`
    salutation: typing.Optional[str]
    #: Optional :class:`str`
    key: typing.Optional[str]
    #: Optional list of :class:`commercetools.types.StoreKeyReference`
    stores: typing.Optional[typing.List["StoreKeyReference"]]

    def __init__(
        self,
        *,
        id: str,
        version: int,
        created_at: datetime.datetime,
        last_modified_at: datetime.datetime,
        email: str,
        password: str,
        addresses: typing.List["Address"],
        is_email_verified: bool,
        last_modified_by: typing.Optional["LastModifiedBy"] = None,
        created_by: typing.Optional["CreatedBy"] = None,
        customer_number: typing.Optional[str] = None,
        first_name: typing.Optional[str] = None,
        last_name: typing.Optional[str] = None,
        middle_name: typing.Optional[str] = None,
        title: typing.Optional[str] = None,
        date_of_birth: typing.Optional[datetime.date] = None,
        company_name: typing.Optional[str] = None,
        vat_id: typing.Optional[str] = None,
        default_shipping_address_id: typing.Optional[str] = None,
        shipping_address_ids: typing.Optional[typing.List[str]] = None,
        default_billing_address_id: typing.Optional[str] = None,
        billing_address_ids: typing.Optional[typing.List[str]] = None,
        external_id: typing.Optional[str] = None,
        customer_group: typing.Optional["CustomerGroupReference"] = None,
        custom: typing.Optional["CustomFields"] = None,
        locale: typing.Optional[str] = None,
        salutation: typing.Optional[str] = None,
        key: typing.Optional[str] = None,
        stores: typing.Optional[typing.List["StoreKeyReference"]] = None
    ) -> None:
        self.id = id
        self.version = version
        self.created_at = created_at
        self.last_modified_at = last_modified_at
        self.last_modified_by = last_modified_by
        self.created_by = created_by
        self.customer_number = customer_number
        self.email = email
        self.password = password
        self.first_name = first_name
        self.last_name = last_name
        self.middle_name = middle_name
        self.title = title
        self.date_of_birth = date_of_birth
        self.company_name = company_name
        self.vat_id = vat_id
        self.addresses = addresses
        self.default_shipping_address_id = default_shipping_address_id
        self.shipping_address_ids = shipping_address_ids
        self.default_billing_address_id = default_billing_address_id
        self.billing_address_ids = billing_address_ids
        self.is_email_verified = is_email_verified
        self.external_id = external_id
        self.customer_group = customer_group
        self.custom = custom
        self.locale = locale
        self.salutation = salutation
        self.key = key
        self.stores = stores
        super().__init__(
            id=id,
            version=version,
            created_at=created_at,
            last_modified_at=last_modified_at,
        )

    def __repr__(self) -> str:
        return (
            "Customer(id=%r, version=%r, created_at=%r, last_modified_at=%r, last_modified_by=%r, created_by=%r, customer_number=%r, email=%r, password=%r, first_name=%r, last_name=%r, middle_name=%r, title=%r, date_of_birth=%r, company_name=%r, vat_id=%r, addresses=%r, default_shipping_address_id=%r, shipping_address_ids=%r, default_billing_address_id=%r, billing_address_ids=%r, is_email_verified=%r, external_id=%r, customer_group=%r, custom=%r, locale=%r, salutation=%r, key=%r, stores=%r)"
            % (
                self.id,
                self.version,
                self.created_at,
                self.last_modified_at,
                self.last_modified_by,
                self.created_by,
                self.customer_number,
                self.email,
                self.password,
                self.first_name,
                self.last_name,
                self.middle_name,
                self.title,
                self.date_of_birth,
                self.company_name,
                self.vat_id,
                self.addresses,
                self.default_shipping_address_id,
                self.shipping_address_ids,
                self.default_billing_address_id,
                self.billing_address_ids,
                self.is_email_verified,
                self.external_id,
                self.customer_group,
                self.custom,
                self.locale,
                self.salutation,
                self.key,
                self.stores,
            )
        )


class CustomerChangePassword(_BaseType):
    #: :class:`str`
    id: str
    #: :class:`int`
    version: int
    #: :class:`str` `(Named` ``currentPassword`` `in Commercetools)`
    current_password: str
    #: :class:`str` `(Named` ``newPassword`` `in Commercetools)`
    new_password: str

    def __init__(
        self, *, id: str, version: int, current_password: str, new_password: str
    ) -> None:
        self.id = id
        self.version = version
        self.current_password = current_password
        self.new_password = new_password
        super().__init__()

    def __repr__(self) -> str:
        return (
            "CustomerChangePassword(id=%r, version=%r, current_password=%r, new_password=%r)"
            % (self.id, self.version, self.current_password, self.new_password)
        )


class CustomerCreateEmailToken(_BaseType):
    #: :class:`str`
    id: str
    #: Optional :class:`int`
    version: typing.Optional[int]
    #: :class:`int` `(Named` ``ttlMinutes`` `in Commercetools)`
    ttl_minutes: int

    def __init__(
        self, *, id: str, ttl_minutes: int, version: typing.Optional[int] = None
    ) -> None:
        self.id = id
        self.version = version
        self.ttl_minutes = ttl_minutes
        super().__init__()

    def __repr__(self) -> str:
        return "CustomerCreateEmailToken(id=%r, version=%r, ttl_minutes=%r)" % (
            self.id,
            self.version,
            self.ttl_minutes,
        )


class CustomerCreatePasswordResetToken(_BaseType):
    #: :class:`str`
    email: str
    #: Optional :class:`int` `(Named` ``ttlMinutes`` `in Commercetools)`
    ttl_minutes: typing.Optional[int]

    def __init__(self, *, email: str, ttl_minutes: typing.Optional[int] = None) -> None:
        self.email = email
        self.ttl_minutes = ttl_minutes
        super().__init__()

    def __repr__(self) -> str:
        return "CustomerCreatePasswordResetToken(email=%r, ttl_minutes=%r)" % (
            self.email,
            self.ttl_minutes,
        )


class CustomerDraft(_BaseType):
    #: Optional :class:`str` `(Named` ``customerNumber`` `in Commercetools)`
    customer_number: typing.Optional[str]
    #: :class:`str`
    email: str
    #: :class:`str`
    password: str
    #: Optional :class:`str` `(Named` ``firstName`` `in Commercetools)`
    first_name: typing.Optional[str]
    #: Optional :class:`str` `(Named` ``lastName`` `in Commercetools)`
    last_name: typing.Optional[str]
    #: Optional :class:`str` `(Named` ``middleName`` `in Commercetools)`
    middle_name: typing.Optional[str]
    #: Optional :class:`str`
    title: typing.Optional[str]
    #: Optional :class:`str` `(Named` ``anonymousCartId`` `in Commercetools)`
    anonymous_cart_id: typing.Optional[str]
    #: Optional :class:`str` `(Named` ``anonymousId`` `in Commercetools)`
    anonymous_id: typing.Optional[str]
    #: Optional :class:`datetime.date` `(Named` ``dateOfBirth`` `in Commercetools)`
    date_of_birth: typing.Optional[datetime.date]
    #: Optional :class:`str` `(Named` ``companyName`` `in Commercetools)`
    company_name: typing.Optional[str]
    #: Optional :class:`str` `(Named` ``vatId`` `in Commercetools)`
    vat_id: typing.Optional[str]
    #: Optional list of :class:`commercetools.types.Address`
    addresses: typing.Optional[typing.List["Address"]]
    #: Optional :class:`int` `(Named` ``defaultShippingAddress`` `in Commercetools)`
    default_shipping_address: typing.Optional[int]
    #: Optional list of :class:`int` `(Named` ``shippingAddresses`` `in Commercetools)`
    shipping_addresses: typing.Optional[typing.List[int]]
    #: Optional :class:`int` `(Named` ``defaultBillingAddress`` `in Commercetools)`
    default_billing_address: typing.Optional[int]
    #: Optional list of :class:`int` `(Named` ``billingAddresses`` `in Commercetools)`
    billing_addresses: typing.Optional[typing.List[int]]
    #: Optional :class:`bool` `(Named` ``isEmailVerified`` `in Commercetools)`
    is_email_verified: typing.Optional[bool]
    #: Optional :class:`str` `(Named` ``externalId`` `in Commercetools)`
    external_id: typing.Optional[str]
    #: Optional :class:`commercetools.types.CustomerGroupResourceIdentifier` `(Named` ``customerGroup`` `in Commercetools)`
    customer_group: typing.Optional["CustomerGroupResourceIdentifier"]
    #: Optional :class:`commercetools.types.CustomFieldsDraft`
    custom: typing.Optional["CustomFieldsDraft"]
    #: Optional :class:`str`
    locale: typing.Optional[str]
    #: Optional :class:`str`
    salutation: typing.Optional[str]
    #: Optional :class:`str`
    key: typing.Optional[str]
    #: Optional list of :class:`commercetools.types.StoreResourceIdentifier`
    stores: typing.Optional[typing.List["StoreResourceIdentifier"]]

    def __init__(
        self,
        *,
        email: str,
        password: str,
        customer_number: typing.Optional[str] = None,
        first_name: typing.Optional[str] = None,
        last_name: typing.Optional[str] = None,
        middle_name: typing.Optional[str] = None,
        title: typing.Optional[str] = None,
        anonymous_cart_id: typing.Optional[str] = None,
        anonymous_id: typing.Optional[str] = None,
        date_of_birth: typing.Optional[datetime.date] = None,
        company_name: typing.Optional[str] = None,
        vat_id: typing.Optional[str] = None,
        addresses: typing.Optional[typing.List["Address"]] = None,
        default_shipping_address: typing.Optional[int] = None,
        shipping_addresses: typing.Optional[typing.List[int]] = None,
        default_billing_address: typing.Optional[int] = None,
        billing_addresses: typing.Optional[typing.List[int]] = None,
        is_email_verified: typing.Optional[bool] = None,
        external_id: typing.Optional[str] = None,
        customer_group: typing.Optional["CustomerGroupResourceIdentifier"] = None,
        custom: typing.Optional["CustomFieldsDraft"] = None,
        locale: typing.Optional[str] = None,
        salutation: typing.Optional[str] = None,
        key: typing.Optional[str] = None,
        stores: typing.Optional[typing.List["StoreResourceIdentifier"]] = None
    ) -> None:
        self.customer_number = customer_number
        self.email = email
        self.password = password
        self.first_name = first_name
        self.last_name = last_name
        self.middle_name = middle_name
        self.title = title
        self.anonymous_cart_id = anonymous_cart_id
        self.anonymous_id = anonymous_id
        self.date_of_birth = date_of_birth
        self.company_name = company_name
        self.vat_id = vat_id
        self.addresses = addresses
        self.default_shipping_address = default_shipping_address
        self.shipping_addresses = shipping_addresses
        self.default_billing_address = default_billing_address
        self.billing_addresses = billing_addresses
        self.is_email_verified = is_email_verified
        self.external_id = external_id
        self.customer_group = customer_group
        self.custom = custom
        self.locale = locale
        self.salutation = salutation
        self.key = key
        self.stores = stores
        super().__init__()

    def __repr__(self) -> str:
        return (
            "CustomerDraft(customer_number=%r, email=%r, password=%r, first_name=%r, last_name=%r, middle_name=%r, title=%r, anonymous_cart_id=%r, anonymous_id=%r, date_of_birth=%r, company_name=%r, vat_id=%r, addresses=%r, default_shipping_address=%r, shipping_addresses=%r, default_billing_address=%r, billing_addresses=%r, is_email_verified=%r, external_id=%r, customer_group=%r, custom=%r, locale=%r, salutation=%r, key=%r, stores=%r)"
            % (
                self.customer_number,
                self.email,
                self.password,
                self.first_name,
                self.last_name,
                self.middle_name,
                self.title,
                self.anonymous_cart_id,
                self.anonymous_id,
                self.date_of_birth,
                self.company_name,
                self.vat_id,
                self.addresses,
                self.default_shipping_address,
                self.shipping_addresses,
                self.default_billing_address,
                self.billing_addresses,
                self.is_email_verified,
                self.external_id,
                self.customer_group,
                self.custom,
                self.locale,
                self.salutation,
                self.key,
                self.stores,
            )
        )


class CustomerEmailVerify(_BaseType):
    #: Optional :class:`int`
    version: typing.Optional[int]
    #: :class:`str` `(Named` ``tokenValue`` `in Commercetools)`
    token_value: str

    def __init__(
        self, *, token_value: str, version: typing.Optional[int] = None
    ) -> None:
        self.version = version
        self.token_value = token_value
        super().__init__()

    def __repr__(self) -> str:
        return "CustomerEmailVerify(version=%r, token_value=%r)" % (
            self.version,
            self.token_value,
        )


class CustomerPagedQueryResponse(_BaseType):
    #: :class:`int`
    limit: int
    #: :class:`int`
    count: int
    #: Optional :class:`int`
    total: typing.Optional[int]
    #: :class:`int`
    offset: int
    #: List of :class:`commercetools.types.Customer`
    results: typing.Sequence["Customer"]

    def __init__(
        self,
        *,
        limit: int,
        count: int,
        offset: int,
        results: typing.Sequence["Customer"],
        total: typing.Optional[int] = None
    ) -> None:
        self.limit = limit
        self.count = count
        self.total = total
        self.offset = offset
        self.results = results
        super().__init__()

    def __repr__(self) -> str:
        return (
            "CustomerPagedQueryResponse(limit=%r, count=%r, total=%r, offset=%r, results=%r)"
            % (self.limit, self.count, self.total, self.offset, self.results)
        )


class CustomerReference(Reference):
    #: Optional :class:`commercetools.types.Customer`
    obj: typing.Optional["Customer"]

    def __init__(self, *, id: str, obj: typing.Optional["Customer"] = None) -> None:
        self.obj = obj
        super().__init__(type_id=ReferenceTypeId.CUSTOMER, id=id)

    def __repr__(self) -> str:
        return "CustomerReference(type_id=%r, id=%r, obj=%r)" % (
            self.type_id,
            self.id,
            self.obj,
        )


class CustomerResetPassword(_BaseType):
    #: :class:`str` `(Named` ``tokenValue`` `in Commercetools)`
    token_value: str
    #: :class:`str` `(Named` ``newPassword`` `in Commercetools)`
    new_password: str
    #: Optional :class:`int`
    version: typing.Optional[int]

    def __init__(
        self,
        *,
        token_value: str,
        new_password: str,
        version: typing.Optional[int] = None
    ) -> None:
        self.token_value = token_value
        self.new_password = new_password
        self.version = version
        super().__init__()

    def __repr__(self) -> str:
        return "CustomerResetPassword(token_value=%r, new_password=%r, version=%r)" % (
            self.token_value,
            self.new_password,
            self.version,
        )


class CustomerResourceIdentifier(ResourceIdentifier):
    def __init__(
        self, *, id: typing.Optional[str] = None, key: typing.Optional[str] = None
    ) -> None:
        super().__init__(type_id=ReferenceTypeId.CUSTOMER, id=id, key=key)

    def __repr__(self) -> str:
        return "CustomerResourceIdentifier(type_id=%r, id=%r, key=%r)" % (
            self.type_id,
            self.id,
            self.key,
        )


class CustomerSignInResult(_BaseType):
    #: :class:`commercetools.types.Customer`
    customer: "Customer"
    #: Optional :class:`object`
    cart: typing.Optional[object]

    def __init__(
        self, *, customer: "Customer", cart: typing.Optional[object] = None
    ) -> None:
        self.customer = customer
        self.cart = cart
        super().__init__()

    def __repr__(self) -> str:
        return "CustomerSignInResult(customer=%r, cart=%r)" % (self.customer, self.cart)


class CustomerSignin(_BaseType):
    #: :class:`str`
    email: str
    #: :class:`str`
    password: str
    #: Optional :class:`str` `(Named` ``anonymousCartId`` `in Commercetools)`
    anonymous_cart_id: typing.Optional[str]
    #: Optional :class:`commercetools.types.AnonymousCartSignInMode` `(Named` ``anonymousCartSignInMode`` `in Commercetools)`
    anonymous_cart_sign_in_mode: typing.Optional["AnonymousCartSignInMode"]
    #: Optional :class:`str` `(Named` ``anonymousId`` `in Commercetools)`
    anonymous_id: typing.Optional[str]
    #: Optional :class:`bool` `(Named` ``updateProductData`` `in Commercetools)`
    update_product_data: typing.Optional[bool]

    def __init__(
        self,
        *,
        email: str,
        password: str,
        anonymous_cart_id: typing.Optional[str] = None,
        anonymous_cart_sign_in_mode: typing.Optional["AnonymousCartSignInMode"] = None,
        anonymous_id: typing.Optional[str] = None,
        update_product_data: typing.Optional[bool] = None
    ) -> None:
        self.email = email
        self.password = password
        self.anonymous_cart_id = anonymous_cart_id
        self.anonymous_cart_sign_in_mode = anonymous_cart_sign_in_mode
        self.anonymous_id = anonymous_id
        self.update_product_data = update_product_data
        super().__init__()

    def __repr__(self) -> str:
        return (
            "CustomerSignin(email=%r, password=%r, anonymous_cart_id=%r, anonymous_cart_sign_in_mode=%r, anonymous_id=%r, update_product_data=%r)"
            % (
                self.email,
                self.password,
                self.anonymous_cart_id,
                self.anonymous_cart_sign_in_mode,
                self.anonymous_id,
                self.update_product_data,
            )
        )


class CustomerToken(_BaseType):
    #: :class:`str`
    id: str
    #: :class:`datetime.datetime` `(Named` ``createdAt`` `in Commercetools)`
    created_at: datetime.datetime
    #: Optional :class:`datetime.datetime` `(Named` ``lastModifiedAt`` `in Commercetools)`
    last_modified_at: typing.Optional[datetime.datetime]
    #: :class:`str` `(Named` ``customerId`` `in Commercetools)`
    customer_id: str
    #: :class:`datetime.datetime` `(Named` ``expiresAt`` `in Commercetools)`
    expires_at: datetime.datetime
    #: :class:`str`
    value: str

    def __init__(
        self,
        *,
        id: str,
        created_at: datetime.datetime,
        customer_id: str,
        expires_at: datetime.datetime,
        value: str,
        last_modified_at: typing.Optional[datetime.datetime] = None
    ) -> None:
        self.id = id
        self.created_at = created_at
        self.last_modified_at = last_modified_at
        self.customer_id = customer_id
        self.expires_at = expires_at
        self.value = value
        super().__init__()

    def __repr__(self) -> str:
        return (
            "CustomerToken(id=%r, created_at=%r, last_modified_at=%r, customer_id=%r, expires_at=%r, value=%r)"
            % (
                self.id,
                self.created_at,
                self.last_modified_at,
                self.customer_id,
                self.expires_at,
                self.value,
            )
        )


class CustomerUpdate(_BaseType):
    #: :class:`int`
    version: int
    #: :class:`list`
    actions: list

    def __init__(self, *, version: int, actions: list) -> None:
        self.version = version
        self.actions = actions
        super().__init__()

    def __repr__(self) -> str:
        return "CustomerUpdate(version=%r, actions=%r)" % (self.version, self.actions)


class CustomerUpdateAction(_BaseType):
    #: :class:`str`
    action: str

    def __init__(self, *, action: str) -> None:
        self.action = action
        super().__init__()

    def __repr__(self) -> str:
        return "CustomerUpdateAction(action=%r)" % (self.action,)


class CustomerAddAddressAction(CustomerUpdateAction):
    #: :class:`commercetools.types.Address`
    address: "Address"

    def __init__(self, *, address: "Address") -> None:
        self.address = address
        super().__init__(action="addAddress")

    def __repr__(self) -> str:
        return "CustomerAddAddressAction(action=%r, address=%r)" % (
            self.action,
            self.address,
        )


class CustomerAddBillingAddressIdAction(CustomerUpdateAction):
    #: Optional :class:`str` `(Named` ``addressId`` `in Commercetools)`
    address_id: typing.Optional[str]
    #: Optional :class:`str` `(Named` ``addressKey`` `in Commercetools)`
    address_key: typing.Optional[str]

    def __init__(
        self,
        *,
        address_id: typing.Optional[str] = None,
        address_key: typing.Optional[str] = None
    ) -> None:
        self.address_id = address_id
        self.address_key = address_key
        super().__init__(action="addBillingAddressId")

    def __repr__(self) -> str:
        return (
            "CustomerAddBillingAddressIdAction(action=%r, address_id=%r, address_key=%r)"
            % (self.action, self.address_id, self.address_key)
        )


class CustomerAddShippingAddressIdAction(CustomerUpdateAction):
    #: Optional :class:`str` `(Named` ``addressId`` `in Commercetools)`
    address_id: typing.Optional[str]
    #: Optional :class:`str` `(Named` ``addressKey`` `in Commercetools)`
    address_key: typing.Optional[str]

    def __init__(
        self,
        *,
        address_id: typing.Optional[str] = None,
        address_key: typing.Optional[str] = None
    ) -> None:
        self.address_id = address_id
        self.address_key = address_key
        super().__init__(action="addShippingAddressId")

    def __repr__(self) -> str:
        return (
            "CustomerAddShippingAddressIdAction(action=%r, address_id=%r, address_key=%r)"
            % (self.action, self.address_id, self.address_key)
        )


class CustomerAddStoreAction(CustomerUpdateAction):
    #: :class:`commercetools.types.StoreResourceIdentifier`
    store: "StoreResourceIdentifier"

    def __init__(self, *, store: "StoreResourceIdentifier") -> None:
        self.store = store
        super().__init__(action="addStore")

    def __repr__(self) -> str:
        return "CustomerAddStoreAction(action=%r, store=%r)" % (self.action, self.store)


class CustomerChangeAddressAction(CustomerUpdateAction):
    #: Optional :class:`str` `(Named` ``addressId`` `in Commercetools)`
    address_id: typing.Optional[str]
    #: Optional :class:`str` `(Named` ``addressKey`` `in Commercetools)`
    address_key: typing.Optional[str]
    #: :class:`commercetools.types.Address`
    address: "Address"

    def __init__(
        self,
        *,
        address: "Address",
        address_id: typing.Optional[str] = None,
        address_key: typing.Optional[str] = None
    ) -> None:
        self.address_id = address_id
        self.address_key = address_key
        self.address = address
        super().__init__(action="changeAddress")

    def __repr__(self) -> str:
        return (
            "CustomerChangeAddressAction(action=%r, address_id=%r, address_key=%r, address=%r)"
            % (self.action, self.address_id, self.address_key, self.address)
        )


class CustomerChangeEmailAction(CustomerUpdateAction):
    #: :class:`str`
    email: str

    def __init__(self, *, email: str) -> None:
        self.email = email
        super().__init__(action="changeEmail")

    def __repr__(self) -> str:
        return "CustomerChangeEmailAction(action=%r, email=%r)" % (
            self.action,
            self.email,
        )


class CustomerRemoveAddressAction(CustomerUpdateAction):
    #: Optional :class:`str` `(Named` ``addressId`` `in Commercetools)`
    address_id: typing.Optional[str]
    #: Optional :class:`str` `(Named` ``addressKey`` `in Commercetools)`
    address_key: typing.Optional[str]

    def __init__(
        self,
        *,
        address_id: typing.Optional[str] = None,
        address_key: typing.Optional[str] = None
    ) -> None:
        self.address_id = address_id
        self.address_key = address_key
        super().__init__(action="removeAddress")

    def __repr__(self) -> str:
        return (
            "CustomerRemoveAddressAction(action=%r, address_id=%r, address_key=%r)"
            % (self.action, self.address_id, self.address_key)
        )


class CustomerRemoveBillingAddressIdAction(CustomerUpdateAction):
    #: Optional :class:`str` `(Named` ``addressId`` `in Commercetools)`
    address_id: typing.Optional[str]
    #: Optional :class:`str` `(Named` ``addressKey`` `in Commercetools)`
    address_key: typing.Optional[str]

    def __init__(
        self,
        *,
        address_id: typing.Optional[str] = None,
        address_key: typing.Optional[str] = None
    ) -> None:
        self.address_id = address_id
        self.address_key = address_key
        super().__init__(action="removeBillingAddressId")

    def __repr__(self) -> str:
        return (
            "CustomerRemoveBillingAddressIdAction(action=%r, address_id=%r, address_key=%r)"
            % (self.action, self.address_id, self.address_key)
        )


class CustomerRemoveShippingAddressIdAction(CustomerUpdateAction):
    #: Optional :class:`str` `(Named` ``addressId`` `in Commercetools)`
    address_id: typing.Optional[str]
    #: Optional :class:`str` `(Named` ``addressKey`` `in Commercetools)`
    address_key: typing.Optional[str]

    def __init__(
        self,
        *,
        address_id: typing.Optional[str] = None,
        address_key: typing.Optional[str] = None
    ) -> None:
        self.address_id = address_id
        self.address_key = address_key
        super().__init__(action="removeShippingAddressId")

    def __repr__(self) -> str:
        return (
            "CustomerRemoveShippingAddressIdAction(action=%r, address_id=%r, address_key=%r)"
            % (self.action, self.address_id, self.address_key)
        )


class CustomerRemoveStoreAction(CustomerUpdateAction):
    #: :class:`commercetools.types.StoreResourceIdentifier`
    store: "StoreResourceIdentifier"

    def __init__(self, *, store: "StoreResourceIdentifier") -> None:
        self.store = store
        super().__init__(action="removeStore")

    def __repr__(self) -> str:
        return "CustomerRemoveStoreAction(action=%r, store=%r)" % (
            self.action,
            self.store,
        )


class CustomerSetCompanyNameAction(CustomerUpdateAction):
    #: Optional :class:`str` `(Named` ``companyName`` `in Commercetools)`
    company_name: typing.Optional[str]

    def __init__(self, *, company_name: typing.Optional[str] = None) -> None:
        self.company_name = company_name
        super().__init__(action="setCompanyName")

    def __repr__(self) -> str:
        return "CustomerSetCompanyNameAction(action=%r, company_name=%r)" % (
            self.action,
            self.company_name,
        )


class CustomerSetCustomFieldAction(CustomerUpdateAction):
    #: :class:`str`
    name: str
    #: Optional :class:`typing.Any`
    value: typing.Optional[typing.Any]

    def __init__(self, *, name: str, value: typing.Optional[typing.Any] = None) -> None:
        self.name = name
        self.value = value
        super().__init__(action="setCustomField")

    def __repr__(self) -> str:
        return "CustomerSetCustomFieldAction(action=%r, name=%r, value=%r)" % (
            self.action,
            self.name,
            self.value,
        )


class CustomerSetCustomTypeAction(CustomerUpdateAction):
    #: Optional :class:`commercetools.types.TypeResourceIdentifier`
    type: typing.Optional["TypeResourceIdentifier"]
    #: Optional :class:`commercetools.types.FieldContainer`
    fields: typing.Optional["FieldContainer"]

    def __init__(
        self,
        *,
        type: typing.Optional["TypeResourceIdentifier"] = None,
        fields: typing.Optional["FieldContainer"] = None
    ) -> None:
        self.type = type
        self.fields = fields
        super().__init__(action="setCustomType")

    def __repr__(self) -> str:
        return "CustomerSetCustomTypeAction(action=%r, type=%r, fields=%r)" % (
            self.action,
            self.type,
            self.fields,
        )


class CustomerSetCustomerGroupAction(CustomerUpdateAction):
    #: Optional :class:`commercetools.types.CustomerGroupResourceIdentifier` `(Named` ``customerGroup`` `in Commercetools)`
    customer_group: typing.Optional["CustomerGroupResourceIdentifier"]

    def __init__(
        self,
        *,
        customer_group: typing.Optional["CustomerGroupResourceIdentifier"] = None
    ) -> None:
        self.customer_group = customer_group
        super().__init__(action="setCustomerGroup")

    def __repr__(self) -> str:
        return "CustomerSetCustomerGroupAction(action=%r, customer_group=%r)" % (
            self.action,
            self.customer_group,
        )


class CustomerSetCustomerNumberAction(CustomerUpdateAction):
    #: Optional :class:`str` `(Named` ``customerNumber`` `in Commercetools)`
    customer_number: typing.Optional[str]

    def __init__(self, *, customer_number: typing.Optional[str] = None) -> None:
        self.customer_number = customer_number
        super().__init__(action="setCustomerNumber")

    def __repr__(self) -> str:
        return "CustomerSetCustomerNumberAction(action=%r, customer_number=%r)" % (
            self.action,
            self.customer_number,
        )


class CustomerSetDateOfBirthAction(CustomerUpdateAction):
    #: Optional :class:`datetime.date` `(Named` ``dateOfBirth`` `in Commercetools)`
    date_of_birth: typing.Optional[datetime.date]

    def __init__(self, *, date_of_birth: typing.Optional[datetime.date] = None) -> None:
        self.date_of_birth = date_of_birth
        super().__init__(action="setDateOfBirth")

    def __repr__(self) -> str:
        return "CustomerSetDateOfBirthAction(action=%r, date_of_birth=%r)" % (
            self.action,
            self.date_of_birth,
        )


class CustomerSetDefaultBillingAddressAction(CustomerUpdateAction):
    #: Optional :class:`str` `(Named` ``addressId`` `in Commercetools)`
    address_id: typing.Optional[str]
    #: Optional :class:`str` `(Named` ``addressKey`` `in Commercetools)`
    address_key: typing.Optional[str]

    def __init__(
        self,
        *,
        address_id: typing.Optional[str] = None,
        address_key: typing.Optional[str] = None
    ) -> None:
        self.address_id = address_id
        self.address_key = address_key
        super().__init__(action="setDefaultBillingAddress")

    def __repr__(self) -> str:
        return (
            "CustomerSetDefaultBillingAddressAction(action=%r, address_id=%r, address_key=%r)"
            % (self.action, self.address_id, self.address_key)
        )


class CustomerSetDefaultShippingAddressAction(CustomerUpdateAction):
    #: Optional :class:`str` `(Named` ``addressId`` `in Commercetools)`
    address_id: typing.Optional[str]
    #: Optional :class:`str` `(Named` ``addressKey`` `in Commercetools)`
    address_key: typing.Optional[str]

    def __init__(
        self,
        *,
        address_id: typing.Optional[str] = None,
        address_key: typing.Optional[str] = None
    ) -> None:
        self.address_id = address_id
        self.address_key = address_key
        super().__init__(action="setDefaultShippingAddress")

    def __repr__(self) -> str:
        return (
            "CustomerSetDefaultShippingAddressAction(action=%r, address_id=%r, address_key=%r)"
            % (self.action, self.address_id, self.address_key)
        )


class CustomerSetExternalIdAction(CustomerUpdateAction):
    #: Optional :class:`str` `(Named` ``externalId`` `in Commercetools)`
    external_id: typing.Optional[str]

    def __init__(self, *, external_id: typing.Optional[str] = None) -> None:
        self.external_id = external_id
        super().__init__(action="setExternalId")

    def __repr__(self) -> str:
        return "CustomerSetExternalIdAction(action=%r, external_id=%r)" % (
            self.action,
            self.external_id,
        )


class CustomerSetFirstNameAction(CustomerUpdateAction):
    #: Optional :class:`str` `(Named` ``firstName`` `in Commercetools)`
    first_name: typing.Optional[str]

    def __init__(self, *, first_name: typing.Optional[str] = None) -> None:
        self.first_name = first_name
        super().__init__(action="setFirstName")

    def __repr__(self) -> str:
        return "CustomerSetFirstNameAction(action=%r, first_name=%r)" % (
            self.action,
            self.first_name,
        )


class CustomerSetKeyAction(CustomerUpdateAction):
    #: Optional :class:`str`
    key: typing.Optional[str]

    def __init__(self, *, key: typing.Optional[str] = None) -> None:
        self.key = key
        super().__init__(action="setKey")

    def __repr__(self) -> str:
        return "CustomerSetKeyAction(action=%r, key=%r)" % (self.action, self.key)


class CustomerSetLastNameAction(CustomerUpdateAction):
    #: Optional :class:`str` `(Named` ``lastName`` `in Commercetools)`
    last_name: typing.Optional[str]

    def __init__(self, *, last_name: typing.Optional[str] = None) -> None:
        self.last_name = last_name
        super().__init__(action="setLastName")

    def __repr__(self) -> str:
        return "CustomerSetLastNameAction(action=%r, last_name=%r)" % (
            self.action,
            self.last_name,
        )


class CustomerSetLocaleAction(CustomerUpdateAction):
    #: Optional :class:`str`
    locale: typing.Optional[str]

    def __init__(self, *, locale: typing.Optional[str] = None) -> None:
        self.locale = locale
        super().__init__(action="setLocale")

    def __repr__(self) -> str:
        return "CustomerSetLocaleAction(action=%r, locale=%r)" % (
            self.action,
            self.locale,
        )


class CustomerSetMiddleNameAction(CustomerUpdateAction):
    #: Optional :class:`str` `(Named` ``middleName`` `in Commercetools)`
    middle_name: typing.Optional[str]

    def __init__(self, *, middle_name: typing.Optional[str] = None) -> None:
        self.middle_name = middle_name
        super().__init__(action="setMiddleName")

    def __repr__(self) -> str:
        return "CustomerSetMiddleNameAction(action=%r, middle_name=%r)" % (
            self.action,
            self.middle_name,
        )


class CustomerSetSalutationAction(CustomerUpdateAction):
    #: Optional :class:`str`
    salutation: typing.Optional[str]

    def __init__(self, *, salutation: typing.Optional[str] = None) -> None:
        self.salutation = salutation
        super().__init__(action="setSalutation")

    def __repr__(self) -> str:
        return "CustomerSetSalutationAction(action=%r, salutation=%r)" % (
            self.action,
            self.salutation,
        )


class CustomerSetStoresAction(CustomerUpdateAction):
    #: Optional list of :class:`commercetools.types.StoreResourceIdentifier`
    stores: typing.Optional[typing.List["StoreResourceIdentifier"]]

    def __init__(
        self, *, stores: typing.Optional[typing.List["StoreResourceIdentifier"]] = None
    ) -> None:
        self.stores = stores
        super().__init__(action="setStores")

    def __repr__(self) -> str:
        return "CustomerSetStoresAction(action=%r, stores=%r)" % (
            self.action,
            self.stores,
        )


class CustomerSetTitleAction(CustomerUpdateAction):
    #: Optional :class:`str`
    title: typing.Optional[str]

    def __init__(self, *, title: typing.Optional[str] = None) -> None:
        self.title = title
        super().__init__(action="setTitle")

    def __repr__(self) -> str:
        return "CustomerSetTitleAction(action=%r, title=%r)" % (self.action, self.title)


class CustomerSetVatIdAction(CustomerUpdateAction):
    #: Optional :class:`str` `(Named` ``vatId`` `in Commercetools)`
    vat_id: typing.Optional[str]

    def __init__(self, *, vat_id: typing.Optional[str] = None) -> None:
        self.vat_id = vat_id
        super().__init__(action="setVatId")

    def __repr__(self) -> str:
        return "CustomerSetVatIdAction(action=%r, vat_id=%r)" % (
            self.action,
            self.vat_id,
        )



# DO NOT EDIT! This file is automatically generated
import typing

import marshmallow
from marshmallow import fields

from commercetools._schemas._shipping_method import (
    ShippingMethodDraftSchema,
    ShippingMethodPagedQueryResponseSchema,
    ShippingMethodSchema,
    ShippingMethodUpdateSchema,
)
from commercetools.helpers import OptionalList, RemoveEmptyValuesMixin
from commercetools.types._shipping_method import (
    ShippingMethod,
    ShippingMethodDraft,
    ShippingMethodPagedQueryResponse,
    ShippingMethodUpdate,
    ShippingMethodUpdateAction,
)
from commercetools.typing import OptionalListStr

from . import abstract, traits


class _ShippingMethodQuerySchema(
    traits.ExpandableSchema,
    traits.SortableSchema,
    traits.PagingSchema,
    traits.QuerySchema,
):
    pass


class _ShippingMethodUpdateSchema(traits.ExpandableSchema, traits.VersionedSchema):
    pass


class _ShippingMethodDeleteSchema(traits.VersionedSchema, traits.ExpandableSchema):
    pass


class _ShippingMethodMatching_CartSchema(traits.ExpandableSchema):
    cart_id = OptionalList(fields.String(), data_key="cartId")


class _ShippingMethodMatching_LocationSchema(traits.ExpandableSchema):
    country = OptionalList(fields.String())
    state = OptionalList(fields.String(), required=False)
    currency = OptionalList(fields.String(), required=False)


class _ShippingMethodMatching_OrdereditSchema(
    marshmallow.Schema, RemoveEmptyValuesMixin
):
    order_edit_id = OptionalList(fields.String(), data_key="orderEditId")
    country = OptionalList(fields.String())
    state = OptionalList(fields.String(), required=False)


class ShippingMethodService(abstract.AbstractService):
    """Shipping Methods define where orders can be shipped and what the costs are."""

    def get_by_id(self, id: str, *, expand: OptionalListStr = None) -> ShippingMethod:
        params = self._serialize_params({"expand": expand}, traits.ExpandableSchema)
        return self._client._get(
            endpoint=f"shipping-methods/{id}",
            params=params,
            schema_cls=ShippingMethodSchema,
        )

    def get_by_key(self, key: str, *, expand: OptionalListStr = None) -> ShippingMethod:
        params = self._serialize_params({"expand": expand}, traits.ExpandableSchema)
        return self._client._get(
            endpoint=f"shipping-methods/key={key}",
            params=params,
            schema_cls=ShippingMethodSchema,
        )

    def query(
        self,
        *,
        expand: OptionalListStr = None,
        sort: OptionalListStr = None,
        limit: int = None,
        offset: int = None,
        with_total: bool = None,
        where: OptionalListStr = None,
        predicate_var: typing.Dict[str, str] = None,
    ) -> ShippingMethodPagedQueryResponse:
        """Shipping Methods define where orders can be shipped and what the costs
        are.
        """
        params = self._serialize_params(
            {
                "expand": expand,
                "sort": sort,
                "limit": limit,
                "offset": offset,
                "with_total": with_total,
                "where": where,
                "predicate_var": predicate_var,
            },
            _ShippingMethodQuerySchema,
        )
        return self._client._get(
            endpoint="shipping-methods",
            params=params,
            schema_cls=ShippingMethodPagedQueryResponseSchema,
        )

    def create(
        self, draft: ShippingMethodDraft, *, expand: OptionalListStr = None
    ) -> ShippingMethod:
        """Shipping Methods define where orders can be shipped and what the costs
        are.
        """
        params = self._serialize_params({"expand": expand}, traits.ExpandableSchema)
        return self._client._post(
            endpoint="shipping-methods",
            params=params,
            data_object=draft,
            request_schema_cls=ShippingMethodDraftSchema,
            response_schema_cls=ShippingMethodSchema,
        )

    def update_by_id(
        self,
        id: str,
        version: int,
        actions: typing.List[ShippingMethodUpdateAction],
        *,
        expand: OptionalListStr = None,
        force_update: bool = False,
    ) -> ShippingMethod:
        params = self._serialize_params({"expand": expand}, _ShippingMethodUpdateSchema)
        update_action = ShippingMethodUpdate(version=version, actions=actions)
        return self._client._post(
            endpoint=f"shipping-methods/{id}",
            params=params,
            data_object=update_action,
            request_schema_cls=ShippingMethodUpdateSchema,
            response_schema_cls=ShippingMethodSchema,
            force_update=force_update,
        )

    def update_by_key(
        self,
        key: str,
        version: int,
        actions: typing.List[ShippingMethodUpdateAction],
        *,
        expand: OptionalListStr = None,
        force_update: bool = False,
    ) -> ShippingMethod:
        params = self._serialize_params({"expand": expand}, _ShippingMethodUpdateSchema)
        update_action = ShippingMethodUpdate(version=version, actions=actions)
        return self._client._post(
            endpoint=f"shipping-methods/key={key}",
            params=params,
            data_object=update_action,
            request_schema_cls=ShippingMethodUpdateSchema,
            response_schema_cls=ShippingMethodSchema,
            force_update=force_update,
        )

    def delete_by_id(
        self,
        id: str,
        version: int,
        *,
        expand: OptionalListStr = None,
        force_delete: bool = False,
    ) -> ShippingMethod:
        params = self._serialize_params(
            {"version": version, "expand": expand}, _ShippingMethodDeleteSchema
        )
        return self._client._delete(
            endpoint=f"shipping-methods/{id}",
            params=params,
            response_schema_cls=ShippingMethodSchema,
            force_delete=force_delete,
        )

    def delete_by_key(
        self,
        key: str,
        version: int,
        *,
        expand: OptionalListStr = None,
        force_delete: bool = False,
    ) -> ShippingMethod:
        params = self._serialize_params(
            {"version": version, "expand": expand}, _ShippingMethodDeleteSchema
        )
        return self._client._delete(
            endpoint=f"shipping-methods/key={key}",
            params=params,
            response_schema_cls=ShippingMethodSchema,
            force_delete=force_delete,
        )

    def matching_cart(
        self, cart_id: str, *, expand: OptionalListStr = None
    ) -> ShippingMethodPagedQueryResponse:
        """Get ShippingMethods for a cart
        """
        params = self._serialize_params(
            {"expand": expand, "cart_id": cart_id}, _ShippingMethodMatching_CartSchema
        )
        return self._client._get(
            endpoint="shipping-methods/matching-cart",
            params=params,
            schema_cls=ShippingMethodPagedQueryResponseSchema,
        )

    def matching_location(
        self,
        country: str,
        *,
        expand: OptionalListStr = None,
        state: str = None,
        currency: str = None,
    ) -> ShippingMethodPagedQueryResponse:
        """Get ShippingMethods for a location
        """
        params = self._serialize_params(
            {
                "expand": expand,
                "country": country,
                "state": state,
                "currency": currency,
            },
            _ShippingMethodMatching_LocationSchema,
        )
        return self._client._get(
            endpoint="shipping-methods/matching-location",
            params=params,
            schema_cls=ShippingMethodPagedQueryResponseSchema,
        )

    def matching_orderedit(
        self, order_edit_id: str, country: str, *, state: str = None
    ) -> ShippingMethodPagedQueryResponse:
        """Get ShippingMethods for an order edit
        """
        params = self._serialize_params(
            {"order_edit_id": order_edit_id, "country": country, "state": state},
            _ShippingMethodMatching_OrdereditSchema,
        )
        return self._client._get(
            endpoint="shipping-methods/matching-orderedit",
            params=params,
            schema_cls=ShippingMethodPagedQueryResponseSchema,
        )



from typing import List

from commercetools import types
from commercetools._schemas._project import ProjectSchema, ProjectUpdateSchema
from commercetools.services import abstract

__all__ = ["ProjectService"]


class ProjectService(abstract.AbstractService):
    def get(self) -> types.Order:
        return self._client._get("", {}, ProjectSchema)

    def update(
        self,
        version: int,
        actions: List[types.OrderUpdateAction],
        *,
        force_update: bool = False,
    ) -> types.Order:
        update_action = types.ProjectUpdate(version=version, actions=actions)
        return self._client._post(
            endpoint="",
            params={},
            data_object=update_action,
            request_schema_cls=ProjectUpdateSchema,
            response_schema_cls=ProjectSchema,
            force_update=force_update,
        )



import json
from collections.abc import Collection
from decimal import Decimal
from functools import reduce


class QueryPredicate:
    AND = "AND"
    OR = "OR"

    _operators = {
        "exact": "=",
        "gte": ">=",
        "in": "in",
        "lte": "=<",
        "lt": "<",
        "gt": ">",
        "is_defined": ("is defined", "is not defined"),
        "contains_all": "contains all",
        "contains_any": "contains any",
    }

    def __init__(self, **filters: str) -> None:
        self._siblings = filters.pop("_siblings", [])
        self._filters = filters

    def __str__(self) -> str:
        result = []

        for key, value in self._filters.items():
            fields = key.split("__")
            operator = fields.pop()
            if operator not in self._operators:
                fields.append(operator)
                operator = "exact"

            lhs = fields.pop()
            val = self._clause(lhs, operator, value)
            fields.append(val)
            result.append(reduce(lambda x, y: f"{y}({x})", fields[::-1]))

        val = " AND ".join(result)
        if self._siblings:
            return reduce(
                lambda x, y: f"({x}) {y.connector} ({y.query_predicate})",
                self._siblings,
                val,
            )
        return val

    def __or__(self, other):
        _siblings = self._siblings[:] + [_SiblingQueryPredicate(self.OR, other)]
        return self.__class__(**self._filters, _siblings=_siblings)

    def __and__(self, other):
        _siblings = self._siblings[:] + [_SiblingQueryPredicate(self.AND, other)]
        return self.__class__(**self._filters, _siblings=_siblings)

    def _clause(self, lhs, operator, rhs) -> str:
        assert operator in self._operators

        if isinstance(rhs, dict):
            rhs = self.__class__(**rhs)
            return "%s(%s)" % (lhs, rhs)

        if isinstance(rhs, self.__class__):
            return "%s(%s)" % (lhs, rhs)

        op = self._operators[operator]
        if isinstance(op, tuple):
            return "%s %s" % (lhs, op[0 if rhs else 1])
        else:
            rhs = self._escape_value(rhs)
            return "%s %s %s" % (lhs, op, rhs)

    def _escape_value(self, value) -> str:
        if isinstance(value, self.__class__):
            return "(%s)" % value
        if isinstance(value, Decimal):
            return str(value)
        if not isinstance(value, str) and isinstance(value, Collection):
            return "(%s)" % (", ".join(self._escape_value(v) for v in value))
        return json.dumps(value)


class _SiblingQueryPredicate:
    def __init__(self, connector: str, query_predicate: QueryPredicate):
        self.connector = connector
        self.query_predicate = query_predicate



import pytest
from requests.exceptions import HTTPError

from commercetools import types


def test_custom_object_get_by_container_and_key(client):
    custom_object = client.custom_objects.create_or_update(
        types.CustomObjectDraft(container="unittest", key="test-object", value=1234)
    )

    assert custom_object.id
    assert custom_object.container == "unittest"
    assert custom_object.key == "test-object"
    assert custom_object.value == 1234

    custom_object = client.custom_objects.get_by_container_and_key(
        "unittest", "test-object"
    )
    assert custom_object.container == "unittest"
    assert custom_object.key == "test-object"
    assert custom_object.value == 1234

    with pytest.raises(HTTPError):
        client.custom_objects.get_by_container_and_key("invalid", "invalid")


def test_custom_object_query(client):
    client.custom_objects.create_or_update(
        types.CustomObjectDraft(container="unittest", key="test-object-1", value=1234)
    )
    client.custom_objects.create_or_update(
        types.CustomObjectDraft(container="unittest", key="test-object-2", value=1234)
    )

    # single sort query
    result = client.custom_objects.query(sort="id asc")
    assert len(result.results) == 2
    assert result.total == 2

    # multiple sort queries
    result = client.custom_objects.query(sort=["id asc", "name asc"])
    assert len(result.results) == 2
    assert result.total == 2


def test_custom_object_update(client):
    """Test the return value of the update methods.

    It doesn't test the actual update itself.
    TODO: See if this is worth testing since we're using a mocking backend
    """
    custom_object = client.custom_objects.create_or_update(
        types.CustomObjectDraft(container="unittest", key="test-object-1", value=1234)
    )
    assert custom_object.key == "test-object-1"

    custom_object = client.custom_objects.create_or_update(
        types.CustomObjectDraft(container="unittest", key="test-object-1", value=2345)
    )
    assert custom_object.key == "test-object-1"


def test_custom_object_query_by_container(client):
    """Test filtering by container."""
    client.custom_objects.create_or_update(
        types.CustomObjectDraft(container="unittest", key="test-object-1", value=1234)
    )
    client.custom_objects.create_or_update(
        types.CustomObjectDraft(container="unittest", key="test-object-2", value=1234)
    )
    client.custom_objects.create_or_update(
        types.CustomObjectDraft(container="unittest2", key="test-object-1", value=1234)
    )

    result = client.custom_objects.query_by_container("unittest")
    assert len(result.results) == 2
    assert result.total == 2



# DO NOT EDIT! This file is automatically generated
import typing

from commercetools._schemas._inventory import (
    InventoryEntryDraftSchema,
    InventoryEntrySchema,
    InventoryEntryUpdateSchema,
    InventoryPagedQueryResponseSchema,
)
from commercetools.helpers import RemoveEmptyValuesMixin
from commercetools.types._inventory import (
    InventoryEntry,
    InventoryEntryDraft,
    InventoryEntryUpdate,
    InventoryEntryUpdateAction,
    InventoryPagedQueryResponse,
)
from commercetools.typing import OptionalListStr

from . import abstract, traits


class _InventoryEntryQuerySchema(
    traits.ExpandableSchema,
    traits.SortableSchema,
    traits.PagingSchema,
    traits.QuerySchema,
):
    pass


class _InventoryEntryUpdateSchema(traits.ExpandableSchema, traits.VersionedSchema):
    pass


class _InventoryEntryDeleteSchema(traits.VersionedSchema, traits.ExpandableSchema):
    pass


class InventoryEntryService(abstract.AbstractService):
    """Inventory allows you to track stock quantities."""

    def get_by_id(self, id: str, *, expand: OptionalListStr = None) -> InventoryEntry:
        params = self._serialize_params({"expand": expand}, traits.ExpandableSchema)
        return self._client._get(
            endpoint=f"inventory/{id}", params=params, schema_cls=InventoryEntrySchema
        )

    def query(
        self,
        *,
        expand: OptionalListStr = None,
        sort: OptionalListStr = None,
        limit: int = None,
        offset: int = None,
        with_total: bool = None,
        where: OptionalListStr = None,
        predicate_var: typing.Dict[str, str] = None,
    ) -> InventoryPagedQueryResponse:
        """Inventory allows you to track stock quantities.
        """
        params = self._serialize_params(
            {
                "expand": expand,
                "sort": sort,
                "limit": limit,
                "offset": offset,
                "with_total": with_total,
                "where": where,
                "predicate_var": predicate_var,
            },
            _InventoryEntryQuerySchema,
        )
        return self._client._get(
            endpoint="inventory",
            params=params,
            schema_cls=InventoryPagedQueryResponseSchema,
        )

    def create(
        self, draft: InventoryEntryDraft, *, expand: OptionalListStr = None
    ) -> InventoryEntry:
        """Inventory allows you to track stock quantities.
        """
        params = self._serialize_params({"expand": expand}, traits.ExpandableSchema)
        return self._client._post(
            endpoint="inventory",
            params=params,
            data_object=draft,
            request_schema_cls=InventoryEntryDraftSchema,
            response_schema_cls=InventoryEntrySchema,
        )

    def update_by_id(
        self,
        id: str,
        version: int,
        actions: typing.List[InventoryEntryUpdateAction],
        *,
        expand: OptionalListStr = None,
        force_update: bool = False,
    ) -> InventoryEntry:
        params = self._serialize_params({"expand": expand}, _InventoryEntryUpdateSchema)
        update_action = InventoryEntryUpdate(version=version, actions=actions)
        return self._client._post(
            endpoint=f"inventory/{id}",
            params=params,
            data_object=update_action,
            request_schema_cls=InventoryEntryUpdateSchema,
            response_schema_cls=InventoryEntrySchema,
            force_update=force_update,
        )

    def delete_by_id(
        self,
        id: str,
        version: int,
        *,
        expand: OptionalListStr = None,
        force_delete: bool = False,
    ) -> InventoryEntry:
        params = self._serialize_params(
            {"version": version, "expand": expand}, _InventoryEntryDeleteSchema
        )
        return self._client._delete(
            endpoint=f"inventory/{id}",
            params=params,
            response_schema_cls=InventoryEntrySchema,
            force_delete=force_delete,
        )



import copy
import datetime
import typing
import uuid

from commercetools import types
from commercetools._schemas._payment import (
    PaymentDraftSchema,
    PaymentPagedQueryResponseSchema,
    PaymentSchema,
    PaymentUpdateSchema,
    TransactionSchema,
)
from commercetools._schemas._type import CustomFieldsSchema
from commercetools.testing import utils
from commercetools.testing.abstract import BaseModel, ServiceBackend


class PaymentsModel(BaseModel):
    _primary_type_name = "payment"
    _resource_schema = PaymentSchema
    _unique_values = ["key"]

    def _create_from_draft(
        self, draft: types.PaymentDraft, id: typing.Optional[str] = None
    ) -> types.Payment:
        object_id = str(uuid.UUID(id) if id is not None else uuid.uuid4())
        payment = types.Payment(
            id=str(object_id),
            key=draft.key,
            version=1,
            created_at=datetime.datetime.now(datetime.timezone.utc),
            last_modified_at=datetime.datetime.now(datetime.timezone.utc),
            customer=draft.customer,
            amount_authorized=draft.amount_authorized,
            amount_paid=utils._money_to_typed(draft.amount_paid),
            amount_planned=utils._money_to_typed(draft.amount_planned),
            amount_refunded=utils._money_to_typed(draft.amount_refunded),
            anonymous_id=draft.anonymous_id,
            payment_method_info=draft.payment_method_info,
            payment_status=draft.payment_status,
            transactions=[
                self._create_transaction_draft(transaction)
                for transaction in draft.transactions or []
            ],
            custom=utils.create_from_draft(draft.custom),
            interface_interactions=[],
        )
        return payment

    def _create_transaction_draft(
        self, draft: types.TransactionDraft
    ) -> types.Transaction:
        return types.Transaction(
            id=str(uuid.uuid4()),
            timestamp=draft.timestamp,
            type=draft.type,
            amount=utils._money_to_typed(draft.amount),
            interaction_id=draft.interaction_id,
            state=draft.state,
        )


def add_transaction():
    def updater(self, obj, action):
        draft: types.TransactionDraft = getattr(action, "transaction")
        transaction = types.Transaction(
            id=str(uuid.uuid4()),
            timestamp=draft.timestamp,
            type=draft.type,
            amount=utils._money_to_typed(draft.amount),
            interaction_id=draft.interaction_id,
            state=draft.state,
        )
        transaction = TransactionSchema().dump(transaction)
        if not obj["transactions"]:
            obj["transactions"] = []
        new = copy.deepcopy(obj)
        new["transactions"].append(transaction)
        return new

    return updater


def change_transaction_state():
    def updater(self, obj, action):
        transaction_id = getattr(action, "transaction_id")
        transaction_state = getattr(action, "state")
        for index, transaction in enumerate(obj["transactions"]):
            if transaction["id"] == transaction_id:
                new = copy.deepcopy(obj)
                new["transactions"][index]["state"] = transaction_state.value
                return new
        raise ValueError("Could not find transaction with id %s" % transaction_id)

    return updater


def change_transaction_interaction_id():
    def updater(self, obj, action):
        transaction_id = getattr(action, "transaction_id")
        interaction_id = getattr(action, "interaction_id")
        for index, transaction in enumerate(obj["transactions"]):
            if transaction["id"] == transaction_id:
                new = copy.deepcopy(obj)
                new["transactions"][index]["interactionId"] = interaction_id
                return new
        raise ValueError("Could not find transaction with id %s" % transaction_id)

    return updater


def add_interface_interaction():
    def updater(self, obj, action):
        self.model._storage.get_by_resource_identifier(action.type)

        value = types.CustomFields(type=action.type, fields=getattr(action, "fields"))
        value = CustomFieldsSchema().dump(value)
        if not obj["interfaceInteractions"]:
            obj["interfaceInteractions"] = []
        if value not in obj["interfaceInteractions"]:
            new = copy.deepcopy(obj)
            new["interfaceInteractions"].append(value)
            return new
        return obj

    return updater


class PaymentsBackend(ServiceBackend):
    service_path = "payments"
    model_class = PaymentsModel
    _schema_draft = PaymentDraftSchema
    _schema_update = PaymentUpdateSchema
    _schema_query_response = PaymentPagedQueryResponseSchema

    def urls(self):
        return [
            ("^$", "GET", self.query),
            ("^$", "POST", self.create),
            ("^key=(?P<key>[^/]+)$", "GET", self.get_by_key),
            ("^(?P<id>[^/]+)$", "GET", self.get_by_id),
            ("^key=(?P<key>[^/]+)$", "POST", self.update_by_key),
            ("^(?P<id>[^/]+)$", "POST", self.update_by_id),
            ("^(?P<id>[^/]+)$", "DELETE", self.delete_by_id),
        ]

    _actions = {
        "addInterfaceInteraction": add_interface_interaction(),
        "changeTransactionInteractionId": change_transaction_interaction_id(),
        "addTransaction": add_transaction(),
        "changeTransactionState": change_transaction_state(),
    }



# DO NOT EDIT! This file is automatically generated
import marshmallow
import marshmallow_enum

from commercetools import helpers, types
from commercetools._schemas._common import (
    BaseResourceSchema,
    LocalizedStringField,
    ReferenceSchema,
    ResourceIdentifierSchema,
)

__all__ = [
    "StateAddRolesActionSchema",
    "StateChangeInitialActionSchema",
    "StateChangeKeyActionSchema",
    "StateChangeTypeActionSchema",
    "StateDraftSchema",
    "StatePagedQueryResponseSchema",
    "StateReferenceSchema",
    "StateRemoveRolesActionSchema",
    "StateResourceIdentifierSchema",
    "StateSchema",
    "StateSetDescriptionActionSchema",
    "StateSetNameActionSchema",
    "StateSetRolesActionSchema",
    "StateSetTransitionsActionSchema",
    "StateUpdateActionSchema",
    "StateUpdateSchema",
]


class StateDraftSchema(marshmallow.Schema):
    """Marshmallow schema for :class:`commercetools.types.StateDraft`."""

    key = marshmallow.fields.String(allow_none=True)
    type = marshmallow_enum.EnumField(types.StateTypeEnum, by_value=True)
    name = LocalizedStringField(allow_none=True, missing=None)
    description = LocalizedStringField(allow_none=True, missing=None)
    initial = marshmallow.fields.Bool(allow_none=True, missing=None)
    roles = marshmallow.fields.List(
        marshmallow_enum.EnumField(types.StateRoleEnum, by_value=True), missing=None
    )
    transitions = helpers.LazyNestedField(
        nested="commercetools._schemas._state.StateResourceIdentifierSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        many=True,
        missing=None,
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        return types.StateDraft(**data)


class StatePagedQueryResponseSchema(marshmallow.Schema):
    """Marshmallow schema for :class:`commercetools.types.StatePagedQueryResponse`."""

    limit = marshmallow.fields.Integer(allow_none=True)
    count = marshmallow.fields.Integer(allow_none=True)
    total = marshmallow.fields.Integer(allow_none=True, missing=None)
    offset = marshmallow.fields.Integer(allow_none=True)
    results = helpers.LazyNestedField(
        nested="commercetools._schemas._state.StateSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        many=True,
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        return types.StatePagedQueryResponse(**data)


class StateReferenceSchema(ReferenceSchema):
    """Marshmallow schema for :class:`commercetools.types.StateReference`."""

    obj = helpers.LazyNestedField(
        nested="commercetools._schemas._state.StateSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["type_id"]
        return types.StateReference(**data)


class StateResourceIdentifierSchema(ResourceIdentifierSchema):
    """Marshmallow schema for :class:`commercetools.types.StateResourceIdentifier`."""

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["type_id"]
        return types.StateResourceIdentifier(**data)


class StateSchema(BaseResourceSchema):
    """Marshmallow schema for :class:`commercetools.types.State`."""

    id = marshmallow.fields.String(allow_none=True)
    version = marshmallow.fields.Integer(allow_none=True)
    created_at = marshmallow.fields.DateTime(allow_none=True, data_key="createdAt")
    last_modified_at = marshmallow.fields.DateTime(
        allow_none=True, data_key="lastModifiedAt"
    )
    last_modified_by = helpers.LazyNestedField(
        nested="commercetools._schemas._common.LastModifiedBySchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
        data_key="lastModifiedBy",
    )
    created_by = helpers.LazyNestedField(
        nested="commercetools._schemas._common.CreatedBySchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
        data_key="createdBy",
    )
    key = marshmallow.fields.String(allow_none=True)
    type = marshmallow_enum.EnumField(types.StateTypeEnum, by_value=True)
    name = LocalizedStringField(allow_none=True, missing=None)
    description = LocalizedStringField(allow_none=True, missing=None)
    initial = marshmallow.fields.Bool(allow_none=True)
    built_in = marshmallow.fields.Bool(allow_none=True, data_key="builtIn")
    roles = marshmallow.fields.List(
        marshmallow_enum.EnumField(types.StateRoleEnum, by_value=True), missing=None
    )
    transitions = helpers.LazyNestedField(
        nested="commercetools._schemas._state.StateReferenceSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        many=True,
        missing=None,
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        return types.State(**data)


class StateUpdateActionSchema(marshmallow.Schema):
    """Marshmallow schema for :class:`commercetools.types.StateUpdateAction`."""

    action = marshmallow.fields.String(allow_none=True)

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.StateUpdateAction(**data)


class StateUpdateSchema(marshmallow.Schema):
    """Marshmallow schema for :class:`commercetools.types.StateUpdate`."""

    version = marshmallow.fields.Integer(allow_none=True)
    actions = marshmallow.fields.List(
        helpers.Discriminator(
            discriminator_field=("action", "action"),
            discriminator_schemas={
                "addRoles": "commercetools._schemas._state.StateAddRolesActionSchema",
                "changeInitial": "commercetools._schemas._state.StateChangeInitialActionSchema",
                "changeKey": "commercetools._schemas._state.StateChangeKeyActionSchema",
                "changeType": "commercetools._schemas._state.StateChangeTypeActionSchema",
                "removeRoles": "commercetools._schemas._state.StateRemoveRolesActionSchema",
                "setDescription": "commercetools._schemas._state.StateSetDescriptionActionSchema",
                "setName": "commercetools._schemas._state.StateSetNameActionSchema",
                "setRoles": "commercetools._schemas._state.StateSetRolesActionSchema",
                "setTransitions": "commercetools._schemas._state.StateSetTransitionsActionSchema",
            },
            unknown=marshmallow.EXCLUDE,
            allow_none=True,
        ),
        allow_none=True,
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        return types.StateUpdate(**data)


class StateAddRolesActionSchema(StateUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.StateAddRolesAction`."""

    roles = marshmallow.fields.List(
        marshmallow_enum.EnumField(types.StateRoleEnum, by_value=True)
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.StateAddRolesAction(**data)


class StateChangeInitialActionSchema(StateUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.StateChangeInitialAction`."""

    initial = marshmallow.fields.Bool(allow_none=True)

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.StateChangeInitialAction(**data)


class StateChangeKeyActionSchema(StateUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.StateChangeKeyAction`."""

    key = marshmallow.fields.String(allow_none=True)

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.StateChangeKeyAction(**data)


class StateChangeTypeActionSchema(StateUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.StateChangeTypeAction`."""

    type = marshmallow_enum.EnumField(types.StateTypeEnum, by_value=True)

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.StateChangeTypeAction(**data)


class StateRemoveRolesActionSchema(StateUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.StateRemoveRolesAction`."""

    roles = marshmallow.fields.List(
        marshmallow_enum.EnumField(types.StateRoleEnum, by_value=True)
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.StateRemoveRolesAction(**data)


class StateSetDescriptionActionSchema(StateUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.StateSetDescriptionAction`."""

    description = LocalizedStringField(allow_none=True)

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.StateSetDescriptionAction(**data)


class StateSetNameActionSchema(StateUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.StateSetNameAction`."""

    name = LocalizedStringField(allow_none=True)

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.StateSetNameAction(**data)


class StateSetRolesActionSchema(StateUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.StateSetRolesAction`."""

    roles = marshmallow.fields.List(
        marshmallow_enum.EnumField(types.StateRoleEnum, by_value=True)
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.StateSetRolesAction(**data)


class StateSetTransitionsActionSchema(StateUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.StateSetTransitionsAction`."""

    transitions = helpers.LazyNestedField(
        nested="commercetools._schemas._state.StateResourceIdentifierSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        many=True,
        missing=None,
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.StateSetTransitionsAction(**data)



# DO NOT EDIT! This file is automatically generated
import marshmallow
import marshmallow_enum

from commercetools import helpers, types
from commercetools._schemas._common import (
    BaseResourceSchema,
    LocalizedStringField,
    ReferenceSchema,
    ResourceIdentifierSchema,
)
from commercetools._schemas._type import FieldContainerField

__all__ = [
    "PaymentAddInterfaceInteractionActionSchema",
    "PaymentAddTransactionActionSchema",
    "PaymentChangeAmountPlannedActionSchema",
    "PaymentChangeTransactionInteractionIdActionSchema",
    "PaymentChangeTransactionStateActionSchema",
    "PaymentChangeTransactionTimestampActionSchema",
    "PaymentDraftSchema",
    "PaymentMethodInfoSchema",
    "PaymentPagedQueryResponseSchema",
    "PaymentReferenceSchema",
    "PaymentResourceIdentifierSchema",
    "PaymentSchema",
    "PaymentSetAmountPaidActionSchema",
    "PaymentSetAmountRefundedActionSchema",
    "PaymentSetAnonymousIdActionSchema",
    "PaymentSetAuthorizationActionSchema",
    "PaymentSetCustomFieldActionSchema",
    "PaymentSetCustomTypeActionSchema",
    "PaymentSetCustomerActionSchema",
    "PaymentSetExternalIdActionSchema",
    "PaymentSetInterfaceIdActionSchema",
    "PaymentSetKeyActionSchema",
    "PaymentSetMethodInfoInterfaceActionSchema",
    "PaymentSetMethodInfoMethodActionSchema",
    "PaymentSetMethodInfoNameActionSchema",
    "PaymentSetStatusInterfaceCodeActionSchema",
    "PaymentSetStatusInterfaceTextActionSchema",
    "PaymentStatusDraftSchema",
    "PaymentStatusSchema",
    "PaymentTransitionStateActionSchema",
    "PaymentUpdateActionSchema",
    "PaymentUpdateSchema",
    "TransactionDraftSchema",
    "TransactionSchema",
]


class PaymentDraftSchema(marshmallow.Schema):
    """Marshmallow schema for :class:`commercetools.types.PaymentDraft`."""

    customer = helpers.LazyNestedField(
        nested="commercetools._schemas._customer.CustomerResourceIdentifierSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
    )
    anonymous_id = marshmallow.fields.String(
        allow_none=True, missing=None, data_key="anonymousId"
    )
    external_id = marshmallow.fields.String(
        allow_none=True, missing=None, data_key="externalId"
    )
    interface_id = marshmallow.fields.String(
        allow_none=True, missing=None, data_key="interfaceId"
    )
    amount_planned = helpers.LazyNestedField(
        nested="commercetools._schemas._common.MoneySchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        data_key="amountPlanned",
    )
    amount_authorized = helpers.LazyNestedField(
        nested="commercetools._schemas._common.MoneySchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
        data_key="amountAuthorized",
    )
    authorized_until = marshmallow.fields.String(
        allow_none=True, missing=None, data_key="authorizedUntil"
    )
    amount_paid = helpers.LazyNestedField(
        nested="commercetools._schemas._common.MoneySchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
        data_key="amountPaid",
    )
    amount_refunded = helpers.LazyNestedField(
        nested="commercetools._schemas._common.MoneySchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
        data_key="amountRefunded",
    )
    payment_method_info = helpers.LazyNestedField(
        nested="commercetools._schemas._payment.PaymentMethodInfoSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
        data_key="paymentMethodInfo",
    )
    payment_status = helpers.LazyNestedField(
        nested="commercetools._schemas._payment.PaymentStatusDraftSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
        data_key="paymentStatus",
    )
    transactions = helpers.LazyNestedField(
        nested="commercetools._schemas._payment.TransactionDraftSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        many=True,
        missing=None,
    )
    interface_interactions = helpers.LazyNestedField(
        nested="commercetools._schemas._type.CustomFieldsDraftSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        many=True,
        missing=None,
        data_key="interfaceInteractions",
    )
    custom = helpers.LazyNestedField(
        nested="commercetools._schemas._type.CustomFieldsDraftSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
    )
    key = marshmallow.fields.String(allow_none=True, missing=None)

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        return types.PaymentDraft(**data)


class PaymentMethodInfoSchema(marshmallow.Schema):
    """Marshmallow schema for :class:`commercetools.types.PaymentMethodInfo`."""

    payment_interface = marshmallow.fields.String(
        allow_none=True, missing=None, data_key="paymentInterface"
    )
    method = marshmallow.fields.String(allow_none=True, missing=None)
    name = LocalizedStringField(allow_none=True, missing=None)

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        return types.PaymentMethodInfo(**data)


class PaymentPagedQueryResponseSchema(marshmallow.Schema):
    """Marshmallow schema for :class:`commercetools.types.PaymentPagedQueryResponse`."""

    limit = marshmallow.fields.Integer(allow_none=True)
    count = marshmallow.fields.Integer(allow_none=True)
    total = marshmallow.fields.Integer(allow_none=True, missing=None)
    offset = marshmallow.fields.Integer(allow_none=True)
    results = helpers.LazyNestedField(
        nested="commercetools._schemas._payment.PaymentSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        many=True,
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        return types.PaymentPagedQueryResponse(**data)


class PaymentReferenceSchema(ReferenceSchema):
    """Marshmallow schema for :class:`commercetools.types.PaymentReference`."""

    obj = helpers.LazyNestedField(
        nested="commercetools._schemas._payment.PaymentSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["type_id"]
        return types.PaymentReference(**data)


class PaymentResourceIdentifierSchema(ResourceIdentifierSchema):
    """Marshmallow schema for :class:`commercetools.types.PaymentResourceIdentifier`."""

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["type_id"]
        return types.PaymentResourceIdentifier(**data)


class PaymentSchema(BaseResourceSchema):
    """Marshmallow schema for :class:`commercetools.types.Payment`."""

    id = marshmallow.fields.String(allow_none=True)
    version = marshmallow.fields.Integer(allow_none=True)
    created_at = marshmallow.fields.DateTime(allow_none=True, data_key="createdAt")
    last_modified_at = marshmallow.fields.DateTime(
        allow_none=True, data_key="lastModifiedAt"
    )
    last_modified_by = helpers.LazyNestedField(
        nested="commercetools._schemas._common.LastModifiedBySchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
        data_key="lastModifiedBy",
    )
    created_by = helpers.LazyNestedField(
        nested="commercetools._schemas._common.CreatedBySchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
        data_key="createdBy",
    )
    customer = helpers.LazyNestedField(
        nested="commercetools._schemas._customer.CustomerReferenceSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
    )
    anonymous_id = marshmallow.fields.String(
        allow_none=True, missing=None, data_key="anonymousId"
    )
    external_id = marshmallow.fields.String(
        allow_none=True, missing=None, data_key="externalId"
    )
    interface_id = marshmallow.fields.String(
        allow_none=True, missing=None, data_key="interfaceId"
    )
    amount_planned = helpers.Discriminator(
        discriminator_field=("type", "type"),
        discriminator_schemas={
            "centPrecision": "commercetools._schemas._common.CentPrecisionMoneySchema",
            "highPrecision": "commercetools._schemas._common.HighPrecisionMoneySchema",
        },
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        data_key="amountPlanned",
    )
    amount_authorized = helpers.Discriminator(
        discriminator_field=("type", "type"),
        discriminator_schemas={
            "centPrecision": "commercetools._schemas._common.CentPrecisionMoneySchema",
            "highPrecision": "commercetools._schemas._common.HighPrecisionMoneySchema",
        },
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
        data_key="amountAuthorized",
    )
    authorized_until = marshmallow.fields.String(
        allow_none=True, missing=None, data_key="authorizedUntil"
    )
    amount_paid = helpers.Discriminator(
        discriminator_field=("type", "type"),
        discriminator_schemas={
            "centPrecision": "commercetools._schemas._common.CentPrecisionMoneySchema",
            "highPrecision": "commercetools._schemas._common.HighPrecisionMoneySchema",
        },
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
        data_key="amountPaid",
    )
    amount_refunded = helpers.Discriminator(
        discriminator_field=("type", "type"),
        discriminator_schemas={
            "centPrecision": "commercetools._schemas._common.CentPrecisionMoneySchema",
            "highPrecision": "commercetools._schemas._common.HighPrecisionMoneySchema",
        },
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
        data_key="amountRefunded",
    )
    payment_method_info = helpers.LazyNestedField(
        nested="commercetools._schemas._payment.PaymentMethodInfoSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        data_key="paymentMethodInfo",
    )
    payment_status = helpers.LazyNestedField(
        nested="commercetools._schemas._payment.PaymentStatusSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        data_key="paymentStatus",
    )
    transactions = helpers.LazyNestedField(
        nested="commercetools._schemas._payment.TransactionSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        many=True,
    )
    interface_interactions = helpers.LazyNestedField(
        nested="commercetools._schemas._type.CustomFieldsSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        many=True,
        data_key="interfaceInteractions",
    )
    custom = helpers.LazyNestedField(
        nested="commercetools._schemas._type.CustomFieldsSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
    )
    key = marshmallow.fields.String(allow_none=True, missing=None)

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        return types.Payment(**data)


class PaymentStatusDraftSchema(marshmallow.Schema):
    """Marshmallow schema for :class:`commercetools.types.PaymentStatusDraft`."""

    interface_code = marshmallow.fields.String(
        allow_none=True, missing=None, data_key="interfaceCode"
    )
    interface_text = marshmallow.fields.String(
        allow_none=True, missing=None, data_key="interfaceText"
    )
    state = helpers.LazyNestedField(
        nested="commercetools._schemas._state.StateResourceIdentifierSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        return types.PaymentStatusDraft(**data)


class PaymentStatusSchema(marshmallow.Schema):
    """Marshmallow schema for :class:`commercetools.types.PaymentStatus`."""

    interface_code = marshmallow.fields.String(
        allow_none=True, missing=None, data_key="interfaceCode"
    )
    interface_text = marshmallow.fields.String(
        allow_none=True, missing=None, data_key="interfaceText"
    )
    state = helpers.LazyNestedField(
        nested="commercetools._schemas._state.StateReferenceSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        return types.PaymentStatus(**data)


class PaymentUpdateActionSchema(marshmallow.Schema):
    """Marshmallow schema for :class:`commercetools.types.PaymentUpdateAction`."""

    action = marshmallow.fields.String(allow_none=True)

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.PaymentUpdateAction(**data)


class PaymentUpdateSchema(marshmallow.Schema):
    """Marshmallow schema for :class:`commercetools.types.PaymentUpdate`."""

    version = marshmallow.fields.Integer(allow_none=True)
    actions = marshmallow.fields.List(
        helpers.Discriminator(
            discriminator_field=("action", "action"),
            discriminator_schemas={
                "addInterfaceInteraction": "commercetools._schemas._payment.PaymentAddInterfaceInteractionActionSchema",
                "addTransaction": "commercetools._schemas._payment.PaymentAddTransactionActionSchema",
                "changeAmountPlanned": "commercetools._schemas._payment.PaymentChangeAmountPlannedActionSchema",
                "changeTransactionInteractionId": "commercetools._schemas._payment.PaymentChangeTransactionInteractionIdActionSchema",
                "changeTransactionState": "commercetools._schemas._payment.PaymentChangeTransactionStateActionSchema",
                "changeTransactionTimestamp": "commercetools._schemas._payment.PaymentChangeTransactionTimestampActionSchema",
                "setAmountPaid": "commercetools._schemas._payment.PaymentSetAmountPaidActionSchema",
                "setAmountRefunded": "commercetools._schemas._payment.PaymentSetAmountRefundedActionSchema",
                "setAnonymousId": "commercetools._schemas._payment.PaymentSetAnonymousIdActionSchema",
                "setAuthorization": "commercetools._schemas._payment.PaymentSetAuthorizationActionSchema",
                "setCustomField": "commercetools._schemas._payment.PaymentSetCustomFieldActionSchema",
                "setCustomType": "commercetools._schemas._payment.PaymentSetCustomTypeActionSchema",
                "setCustomer": "commercetools._schemas._payment.PaymentSetCustomerActionSchema",
                "setExternalId": "commercetools._schemas._payment.PaymentSetExternalIdActionSchema",
                "setInterfaceId": "commercetools._schemas._payment.PaymentSetInterfaceIdActionSchema",
                "setKey": "commercetools._schemas._payment.PaymentSetKeyActionSchema",
                "setMethodInfoInterface": "commercetools._schemas._payment.PaymentSetMethodInfoInterfaceActionSchema",
                "setMethodInfoMethod": "commercetools._schemas._payment.PaymentSetMethodInfoMethodActionSchema",
                "setMethodInfoName": "commercetools._schemas._payment.PaymentSetMethodInfoNameActionSchema",
                "setStatusInterfaceCode": "commercetools._schemas._payment.PaymentSetStatusInterfaceCodeActionSchema",
                "setStatusInterfaceText": "commercetools._schemas._payment.PaymentSetStatusInterfaceTextActionSchema",
                "transitionState": "commercetools._schemas._payment.PaymentTransitionStateActionSchema",
            },
            unknown=marshmallow.EXCLUDE,
            allow_none=True,
        ),
        allow_none=True,
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        return types.PaymentUpdate(**data)


class TransactionDraftSchema(marshmallow.Schema):
    """Marshmallow schema for :class:`commercetools.types.TransactionDraft`."""

    timestamp = marshmallow.fields.DateTime(allow_none=True, missing=None)
    type = marshmallow_enum.EnumField(types.TransactionType, by_value=True)
    amount = helpers.LazyNestedField(
        nested="commercetools._schemas._common.MoneySchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
    )
    interaction_id = marshmallow.fields.String(
        allow_none=True, missing=None, data_key="interactionId"
    )
    state = marshmallow_enum.EnumField(
        types.TransactionState, by_value=True, missing=None
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        return types.TransactionDraft(**data)


class TransactionSchema(marshmallow.Schema):
    """Marshmallow schema for :class:`commercetools.types.Transaction`."""

    id = marshmallow.fields.String(allow_none=True)
    timestamp = marshmallow.fields.DateTime(allow_none=True, missing=None)
    type = marshmallow_enum.EnumField(types.TransactionType, by_value=True)
    amount = helpers.Discriminator(
        discriminator_field=("type", "type"),
        discriminator_schemas={
            "centPrecision": "commercetools._schemas._common.CentPrecisionMoneySchema",
            "highPrecision": "commercetools._schemas._common.HighPrecisionMoneySchema",
        },
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
    )
    interaction_id = marshmallow.fields.String(
        allow_none=True, missing=None, data_key="interactionId"
    )
    state = marshmallow_enum.EnumField(
        types.TransactionState, by_value=True, missing=None
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        return types.Transaction(**data)


class PaymentAddInterfaceInteractionActionSchema(PaymentUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.PaymentAddInterfaceInteractionAction`."""

    type = helpers.LazyNestedField(
        nested="commercetools._schemas._type.TypeResourceIdentifierSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
    )
    fields = FieldContainerField(allow_none=True, missing=None)  # type: ignore

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.PaymentAddInterfaceInteractionAction(**data)


class PaymentAddTransactionActionSchema(PaymentUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.PaymentAddTransactionAction`."""

    transaction = helpers.LazyNestedField(
        nested="commercetools._schemas._payment.TransactionDraftSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.PaymentAddTransactionAction(**data)


class PaymentChangeAmountPlannedActionSchema(PaymentUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.PaymentChangeAmountPlannedAction`."""

    amount = helpers.LazyNestedField(
        nested="commercetools._schemas._common.MoneySchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.PaymentChangeAmountPlannedAction(**data)


class PaymentChangeTransactionInteractionIdActionSchema(PaymentUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.PaymentChangeTransactionInteractionIdAction`."""

    transaction_id = marshmallow.fields.String(
        allow_none=True, data_key="transactionId"
    )
    interaction_id = marshmallow.fields.String(
        allow_none=True, data_key="interactionId"
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.PaymentChangeTransactionInteractionIdAction(**data)


class PaymentChangeTransactionStateActionSchema(PaymentUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.PaymentChangeTransactionStateAction`."""

    transaction_id = marshmallow.fields.String(
        allow_none=True, data_key="transactionId"
    )
    state = marshmallow_enum.EnumField(types.TransactionState, by_value=True)

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.PaymentChangeTransactionStateAction(**data)


class PaymentChangeTransactionTimestampActionSchema(PaymentUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.PaymentChangeTransactionTimestampAction`."""

    transaction_id = marshmallow.fields.String(
        allow_none=True, data_key="transactionId"
    )
    timestamp = marshmallow.fields.DateTime(allow_none=True)

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.PaymentChangeTransactionTimestampAction(**data)


class PaymentSetAmountPaidActionSchema(PaymentUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.PaymentSetAmountPaidAction`."""

    amount = helpers.LazyNestedField(
        nested="commercetools._schemas._common.MoneySchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.PaymentSetAmountPaidAction(**data)


class PaymentSetAmountRefundedActionSchema(PaymentUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.PaymentSetAmountRefundedAction`."""

    amount = helpers.LazyNestedField(
        nested="commercetools._schemas._common.MoneySchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.PaymentSetAmountRefundedAction(**data)


class PaymentSetAnonymousIdActionSchema(PaymentUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.PaymentSetAnonymousIdAction`."""

    anonymous_id = marshmallow.fields.String(
        allow_none=True, missing=None, data_key="anonymousId"
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.PaymentSetAnonymousIdAction(**data)


class PaymentSetAuthorizationActionSchema(PaymentUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.PaymentSetAuthorizationAction`."""

    amount = helpers.LazyNestedField(
        nested="commercetools._schemas._common.MoneySchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
    )
    until = marshmallow.fields.DateTime(allow_none=True, missing=None)

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.PaymentSetAuthorizationAction(**data)


class PaymentSetCustomFieldActionSchema(PaymentUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.PaymentSetCustomFieldAction`."""

    name = marshmallow.fields.String(allow_none=True)
    value = marshmallow.fields.Raw(allow_none=True, missing=None)

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.PaymentSetCustomFieldAction(**data)


class PaymentSetCustomTypeActionSchema(PaymentUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.PaymentSetCustomTypeAction`."""

    type = helpers.LazyNestedField(
        nested="commercetools._schemas._type.TypeResourceIdentifierSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
    )
    fields = FieldContainerField(allow_none=True, missing=None)  # type: ignore

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.PaymentSetCustomTypeAction(**data)


class PaymentSetCustomerActionSchema(PaymentUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.PaymentSetCustomerAction`."""

    customer = helpers.LazyNestedField(
        nested="commercetools._schemas._customer.CustomerResourceIdentifierSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.PaymentSetCustomerAction(**data)


class PaymentSetExternalIdActionSchema(PaymentUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.PaymentSetExternalIdAction`."""

    external_id = marshmallow.fields.String(
        allow_none=True, missing=None, data_key="externalId"
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.PaymentSetExternalIdAction(**data)


class PaymentSetInterfaceIdActionSchema(PaymentUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.PaymentSetInterfaceIdAction`."""

    interface_id = marshmallow.fields.String(allow_none=True, data_key="interfaceId")

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.PaymentSetInterfaceIdAction(**data)


class PaymentSetKeyActionSchema(PaymentUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.PaymentSetKeyAction`."""

    key = marshmallow.fields.String(allow_none=True, missing=None)

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.PaymentSetKeyAction(**data)


class PaymentSetMethodInfoInterfaceActionSchema(PaymentUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.PaymentSetMethodInfoInterfaceAction`."""

    interface = marshmallow.fields.String(allow_none=True)

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.PaymentSetMethodInfoInterfaceAction(**data)


class PaymentSetMethodInfoMethodActionSchema(PaymentUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.PaymentSetMethodInfoMethodAction`."""

    method = marshmallow.fields.String(allow_none=True, missing=None)

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.PaymentSetMethodInfoMethodAction(**data)


class PaymentSetMethodInfoNameActionSchema(PaymentUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.PaymentSetMethodInfoNameAction`."""

    name = LocalizedStringField(allow_none=True, missing=None)

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.PaymentSetMethodInfoNameAction(**data)


class PaymentSetStatusInterfaceCodeActionSchema(PaymentUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.PaymentSetStatusInterfaceCodeAction`."""

    interface_code = marshmallow.fields.String(
        allow_none=True, missing=None, data_key="interfaceCode"
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.PaymentSetStatusInterfaceCodeAction(**data)


class PaymentSetStatusInterfaceTextActionSchema(PaymentUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.PaymentSetStatusInterfaceTextAction`."""

    interface_text = marshmallow.fields.String(
        allow_none=True, data_key="interfaceText"
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.PaymentSetStatusInterfaceTextAction(**data)


class PaymentTransitionStateActionSchema(PaymentUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.PaymentTransitionStateAction`."""

    state = helpers.LazyNestedField(
        nested="commercetools._schemas._state.StateResourceIdentifierSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
    )
    force = marshmallow.fields.Bool(allow_none=True, missing=None)

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.PaymentTransitionStateAction(**data)



from marshmallow import Schema, fields

from commercetools import helpers


class Dummy(Schema):
    items = helpers.OptionalList(fields.String())


def test_optional_list_str():
    data = Dummy().dump({"items": "foobar"})
    assert data["items"] == ["foobar"]


def test_optional_list_items():
    data = Dummy().dump({"items": ["foobar"]})
    assert data["items"] == ["foobar"]



# DO NOT EDIT! This file is automatically generated
import typing

from commercetools._schemas._tax_category import (
    TaxCategoryDraftSchema,
    TaxCategoryPagedQueryResponseSchema,
    TaxCategorySchema,
    TaxCategoryUpdateSchema,
)
from commercetools.helpers import RemoveEmptyValuesMixin
from commercetools.types._tax_category import (
    TaxCategory,
    TaxCategoryDraft,
    TaxCategoryPagedQueryResponse,
    TaxCategoryUpdate,
    TaxCategoryUpdateAction,
)
from commercetools.typing import OptionalListStr

from . import abstract, traits


class _TaxCategoryQuerySchema(
    traits.ExpandableSchema,
    traits.SortableSchema,
    traits.PagingSchema,
    traits.QuerySchema,
):
    pass


class _TaxCategoryUpdateSchema(traits.ExpandableSchema, traits.VersionedSchema):
    pass


class _TaxCategoryDeleteSchema(traits.VersionedSchema, traits.ExpandableSchema):
    pass


class TaxCategoryService(abstract.AbstractService):
    """Tax Categories define how products are to be taxed in different countries."""

    def get_by_id(self, id: str, *, expand: OptionalListStr = None) -> TaxCategory:
        params = self._serialize_params({"expand": expand}, traits.ExpandableSchema)
        return self._client._get(
            endpoint=f"tax-categories/{id}", params=params, schema_cls=TaxCategorySchema
        )

    def get_by_key(self, key: str, *, expand: OptionalListStr = None) -> TaxCategory:
        params = self._serialize_params({"expand": expand}, traits.ExpandableSchema)
        return self._client._get(
            endpoint=f"tax-categories/key={key}",
            params=params,
            schema_cls=TaxCategorySchema,
        )

    def query(
        self,
        *,
        expand: OptionalListStr = None,
        sort: OptionalListStr = None,
        limit: int = None,
        offset: int = None,
        with_total: bool = None,
        where: OptionalListStr = None,
        predicate_var: typing.Dict[str, str] = None,
    ) -> TaxCategoryPagedQueryResponse:
        """Tax Categories define how products are to be taxed in different
        countries.
        """
        params = self._serialize_params(
            {
                "expand": expand,
                "sort": sort,
                "limit": limit,
                "offset": offset,
                "with_total": with_total,
                "where": where,
                "predicate_var": predicate_var,
            },
            _TaxCategoryQuerySchema,
        )
        return self._client._get(
            endpoint="tax-categories",
            params=params,
            schema_cls=TaxCategoryPagedQueryResponseSchema,
        )

    def create(
        self, draft: TaxCategoryDraft, *, expand: OptionalListStr = None
    ) -> TaxCategory:
        """Tax Categories define how products are to be taxed in different
        countries.
        """
        params = self._serialize_params({"expand": expand}, traits.ExpandableSchema)
        return self._client._post(
            endpoint="tax-categories",
            params=params,
            data_object=draft,
            request_schema_cls=TaxCategoryDraftSchema,
            response_schema_cls=TaxCategorySchema,
        )

    def update_by_id(
        self,
        id: str,
        version: int,
        actions: typing.List[TaxCategoryUpdateAction],
        *,
        expand: OptionalListStr = None,
        force_update: bool = False,
    ) -> TaxCategory:
        params = self._serialize_params({"expand": expand}, _TaxCategoryUpdateSchema)
        update_action = TaxCategoryUpdate(version=version, actions=actions)
        return self._client._post(
            endpoint=f"tax-categories/{id}",
            params=params,
            data_object=update_action,
            request_schema_cls=TaxCategoryUpdateSchema,
            response_schema_cls=TaxCategorySchema,
            force_update=force_update,
        )

    def update_by_key(
        self,
        key: str,
        version: int,
        actions: typing.List[TaxCategoryUpdateAction],
        *,
        expand: OptionalListStr = None,
        force_update: bool = False,
    ) -> TaxCategory:
        params = self._serialize_params({"expand": expand}, _TaxCategoryUpdateSchema)
        update_action = TaxCategoryUpdate(version=version, actions=actions)
        return self._client._post(
            endpoint=f"tax-categories/key={key}",
            params=params,
            data_object=update_action,
            request_schema_cls=TaxCategoryUpdateSchema,
            response_schema_cls=TaxCategorySchema,
            force_update=force_update,
        )

    def delete_by_id(
        self,
        id: str,
        version: int,
        *,
        expand: OptionalListStr = None,
        force_delete: bool = False,
    ) -> TaxCategory:
        params = self._serialize_params(
            {"version": version, "expand": expand}, _TaxCategoryDeleteSchema
        )
        return self._client._delete(
            endpoint=f"tax-categories/{id}",
            params=params,
            response_schema_cls=TaxCategorySchema,
            force_delete=force_delete,
        )

    def delete_by_key(
        self,
        key: str,
        version: int,
        *,
        expand: OptionalListStr = None,
        force_delete: bool = False,
    ) -> TaxCategory:
        params = self._serialize_params(
            {"version": version, "expand": expand}, _TaxCategoryDeleteSchema
        )
        return self._client._delete(
            endpoint=f"tax-categories/key={key}",
            params=params,
            response_schema_cls=TaxCategorySchema,
            force_delete=force_delete,
        )



# DO NOT EDIT! This file is automatically generated
import marshmallow

from commercetools import helpers, types
from commercetools._schemas._common import (
    BaseResourceSchema,
    KeyReferenceSchema,
    LocalizedStringField,
    ReferenceSchema,
    ResourceIdentifierSchema,
)

__all__ = [
    "StoreDraftSchema",
    "StoreKeyReferenceSchema",
    "StorePagedQueryResponseSchema",
    "StoreReferenceSchema",
    "StoreResourceIdentifierSchema",
    "StoreSchema",
    "StoreSetLanguagesActionSchema",
    "StoreSetNameActionSchema",
    "StoreUpdateActionSchema",
    "StoreUpdateSchema",
    "StoresAddDistributionChannelsActionSchema",
    "StoresAddSupplyChannelsActionSchema",
    "StoresRemoveDistributionChannelsActionSchema",
    "StoresRemoveSupplyChannelsActionSchema",
    "StoresSetDistributionChannelsActionSchema",
    "StoresSetSupplyChannelsActionSchema",
]


class StoreDraftSchema(marshmallow.Schema):
    """Marshmallow schema for :class:`commercetools.types.StoreDraft`."""

    key = marshmallow.fields.String(allow_none=True)
    name = LocalizedStringField(allow_none=True)
    languages = marshmallow.fields.List(
        marshmallow.fields.String(allow_none=True), missing=None
    )
    distribution_channels = helpers.LazyNestedField(
        nested="commercetools._schemas._channel.ChannelResourceIdentifierSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        many=True,
        missing=None,
        data_key="distributionChannels",
    )
    supply_channels = helpers.LazyNestedField(
        nested="commercetools._schemas._channel.ChannelResourceIdentifierSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        many=True,
        missing=None,
        data_key="supplyChannels",
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        return types.StoreDraft(**data)


class StoreKeyReferenceSchema(KeyReferenceSchema):
    """Marshmallow schema for :class:`commercetools.types.StoreKeyReference`."""

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["type_id"]
        return types.StoreKeyReference(**data)


class StorePagedQueryResponseSchema(marshmallow.Schema):
    """Marshmallow schema for :class:`commercetools.types.StorePagedQueryResponse`."""

    limit = marshmallow.fields.Integer(allow_none=True)
    count = marshmallow.fields.Integer(allow_none=True)
    total = marshmallow.fields.Integer(allow_none=True, missing=None)
    offset = marshmallow.fields.Integer(allow_none=True)
    results = helpers.LazyNestedField(
        nested="commercetools._schemas._store.StoreSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        many=True,
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        return types.StorePagedQueryResponse(**data)


class StoreReferenceSchema(ReferenceSchema):
    """Marshmallow schema for :class:`commercetools.types.StoreReference`."""

    obj = helpers.LazyNestedField(
        nested="commercetools._schemas._store.StoreSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["type_id"]
        return types.StoreReference(**data)


class StoreResourceIdentifierSchema(ResourceIdentifierSchema):
    """Marshmallow schema for :class:`commercetools.types.StoreResourceIdentifier`."""

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["type_id"]
        return types.StoreResourceIdentifier(**data)


class StoreSchema(BaseResourceSchema):
    """Marshmallow schema for :class:`commercetools.types.Store`."""

    id = marshmallow.fields.String(allow_none=True)
    version = marshmallow.fields.Integer(allow_none=True)
    created_at = marshmallow.fields.DateTime(allow_none=True, data_key="createdAt")
    last_modified_at = marshmallow.fields.DateTime(
        allow_none=True, data_key="lastModifiedAt"
    )
    last_modified_by = helpers.LazyNestedField(
        nested="commercetools._schemas._common.LastModifiedBySchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
        data_key="lastModifiedBy",
    )
    created_by = helpers.LazyNestedField(
        nested="commercetools._schemas._common.CreatedBySchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
        data_key="createdBy",
    )
    key = marshmallow.fields.String(allow_none=True)
    name = LocalizedStringField(allow_none=True, missing=None)
    languages = marshmallow.fields.List(
        marshmallow.fields.String(allow_none=True), missing=None
    )
    distribution_channels = helpers.LazyNestedField(
        nested="commercetools._schemas._channel.ChannelReferenceSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        many=True,
        data_key="distributionChannels",
    )
    supply_channels = helpers.LazyNestedField(
        nested="commercetools._schemas._channel.ChannelResourceIdentifierSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        many=True,
        missing=None,
        data_key="supplyChannels",
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        return types.Store(**data)


class StoreUpdateActionSchema(marshmallow.Schema):
    """Marshmallow schema for :class:`commercetools.types.StoreUpdateAction`."""

    action = marshmallow.fields.String(allow_none=True)

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.StoreUpdateAction(**data)


class StoreUpdateSchema(marshmallow.Schema):
    """Marshmallow schema for :class:`commercetools.types.StoreUpdate`."""

    version = marshmallow.fields.Integer(allow_none=True)
    actions = marshmallow.fields.List(
        helpers.Discriminator(
            discriminator_field=("action", "action"),
            discriminator_schemas={
                "setLanguages": "commercetools._schemas._store.StoreSetLanguagesActionSchema",
                "setName": "commercetools._schemas._store.StoreSetNameActionSchema",
                "addDistributionChannel": "commercetools._schemas._store.StoresAddDistributionChannelsActionSchema",
                "addSupplyChannel": "commercetools._schemas._store.StoresAddSupplyChannelsActionSchema",
                "removeDistributionChannel": "commercetools._schemas._store.StoresRemoveDistributionChannelsActionSchema",
                "removeSupplyChannel": "commercetools._schemas._store.StoresRemoveSupplyChannelsActionSchema",
                "setDistributionChannels": "commercetools._schemas._store.StoresSetDistributionChannelsActionSchema",
                "setSupplyChannels": "commercetools._schemas._store.StoresSetSupplyChannelsActionSchema",
            },
            unknown=marshmallow.EXCLUDE,
            allow_none=True,
        ),
        allow_none=True,
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        return types.StoreUpdate(**data)


class StoreSetLanguagesActionSchema(StoreUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.StoreSetLanguagesAction`."""

    languages = marshmallow.fields.List(
        marshmallow.fields.String(allow_none=True), missing=None
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.StoreSetLanguagesAction(**data)


class StoreSetNameActionSchema(StoreUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.StoreSetNameAction`."""

    name = LocalizedStringField(allow_none=True, missing=None)

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.StoreSetNameAction(**data)


class StoresAddDistributionChannelsActionSchema(StoreUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.StoresAddDistributionChannelsAction`."""

    distribution_channel = helpers.LazyNestedField(
        nested="commercetools._schemas._channel.ChannelResourceIdentifierSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        data_key="distributionChannel",
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.StoresAddDistributionChannelsAction(**data)


class StoresAddSupplyChannelsActionSchema(StoreUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.StoresAddSupplyChannelsAction`."""

    supply_channel = helpers.LazyNestedField(
        nested="commercetools._schemas._channel.ChannelResourceIdentifierSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        data_key="supplyChannel",
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.StoresAddSupplyChannelsAction(**data)


class StoresRemoveDistributionChannelsActionSchema(StoreUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.StoresRemoveDistributionChannelsAction`."""

    distribution_channel = helpers.LazyNestedField(
        nested="commercetools._schemas._channel.ChannelResourceIdentifierSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        data_key="distributionChannel",
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.StoresRemoveDistributionChannelsAction(**data)


class StoresRemoveSupplyChannelsActionSchema(StoreUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.StoresRemoveSupplyChannelsAction`."""

    supply_channel = helpers.LazyNestedField(
        nested="commercetools._schemas._channel.ChannelResourceIdentifierSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        data_key="supplyChannel",
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.StoresRemoveSupplyChannelsAction(**data)


class StoresSetDistributionChannelsActionSchema(StoreUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.StoresSetDistributionChannelsAction`."""

    distribution_channels = helpers.LazyNestedField(
        nested="commercetools._schemas._channel.ChannelResourceIdentifierSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        many=True,
        missing=None,
        data_key="distributionChannels",
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.StoresSetDistributionChannelsAction(**data)


class StoresSetSupplyChannelsActionSchema(StoreUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.StoresSetSupplyChannelsAction`."""

    supply_channels = helpers.LazyNestedField(
        nested="commercetools._schemas._channel.ChannelResourceIdentifierSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        many=True,
        missing=None,
        data_key="supplyChannels",
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.StoresSetSupplyChannelsAction(**data)



import typing

from marshmallow.base import SchemaABC

if typing.TYPE_CHECKING:
    from commercetools.protocols import ClientProtocol


class AbstractService:
    def __init__(self, client: "ClientProtocol") -> None:
        self._client = client
        self._schemas: typing.Dict[str, SchemaABC] = {}

    def _serialize_params(self, params, schema) -> typing.Dict[str, str]:
        if schema not in self._schemas:
            self._schemas[schema] = schema()
        return self._schemas[schema].dump(params)



import logging
import typing

import requests
import webob
import webob.exc
from requests.packages.urllib3.response import HTTPResponse
from requests_mock.request import _RequestObjectProxy

from commercetools.testing import BackendRepository

logger = logging.getLogger(__name__)


class HttpAdapter:
    def __init__(self):
        self._matchers: typing.Any = []

    def add_matcher(self, matcher) -> None:
        self._matchers.append(matcher)

    def match(self, request: _RequestObjectProxy) -> typing.Optional[HTTPResponse]:
        for matcher in self._matchers:
            response = matcher(request, skip_port_check=True)
            if response is not None:
                return response
        return None


class Server:
    """Run a real HTTP server with the mocked endpoints."""

    def __init__(self, repository=None):
        self.adapter = HttpAdapter()
        self.repository = repository or BackendRepository()
        self.repository.register(self.adapter)
        self.api_url = "http://localhost:8989"

    def __call__(self, environ, start_response):
        request = self._create_request(environ)
        request_mock = _RequestObjectProxy(request.prepare(), case_sensitive=True)

        if request_mock.body is None:
            request_mock.body = ""

        response = self.adapter.match(request_mock)
        response = self._create_response(response)
        return response(environ, start_response)

    def _create_request(self, environ: dict) -> requests.Request:
        req = webob.Request(environ)
        req.make_body_seekable()
        return requests.Request(
            method=req.method,
            url=req.url,
            headers=req.headers,
            data=req.body,
            params=req.GET,
            cookies=req.cookies,
        )

    def _create_response(self, response) -> webob.Response:
        if response is not None:
            return webob.Response(
                body=response.text,
                charset="utf-8",
                status=response.status_code,
                headerlist=list(response.headers.items()),
            )
        return webob.exc.HTTPNotFound()


if __name__ == "__main__":
    logging.basicConfig(level=logging.DEBUG)
    server = Server()
    from werkzeug.serving import run_simple

    run_simple("0.0.0.0", port=8989, application=server, use_reloader=True)



# DO NOT EDIT! This file is automatically generated
import marshmallow

from commercetools import helpers, types
from commercetools._schemas._common import (
    BaseResourceSchema,
    ReferenceSchema,
    ResourceIdentifierSchema,
)
from commercetools._schemas._type import FieldContainerField

__all__ = [
    "CustomerGroupChangeNameActionSchema",
    "CustomerGroupDraftSchema",
    "CustomerGroupPagedQueryResponseSchema",
    "CustomerGroupReferenceSchema",
    "CustomerGroupResourceIdentifierSchema",
    "CustomerGroupSchema",
    "CustomerGroupSetCustomFieldActionSchema",
    "CustomerGroupSetCustomTypeActionSchema",
    "CustomerGroupSetKeyActionSchema",
    "CustomerGroupUpdateActionSchema",
    "CustomerGroupUpdateSchema",
]


class CustomerGroupDraftSchema(marshmallow.Schema):
    """Marshmallow schema for :class:`commercetools.types.CustomerGroupDraft`."""

    key = marshmallow.fields.String(allow_none=True, missing=None)
    group_name = marshmallow.fields.String(allow_none=True, data_key="groupName")
    custom = helpers.LazyNestedField(
        nested="commercetools._schemas._type.CustomFieldsSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        return types.CustomerGroupDraft(**data)


class CustomerGroupPagedQueryResponseSchema(marshmallow.Schema):
    """Marshmallow schema for :class:`commercetools.types.CustomerGroupPagedQueryResponse`."""

    limit = marshmallow.fields.Integer(allow_none=True)
    count = marshmallow.fields.Integer(allow_none=True)
    total = marshmallow.fields.Integer(allow_none=True, missing=None)
    offset = marshmallow.fields.Integer(allow_none=True)
    results = helpers.LazyNestedField(
        nested="commercetools._schemas._customer_group.CustomerGroupSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        many=True,
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        return types.CustomerGroupPagedQueryResponse(**data)


class CustomerGroupReferenceSchema(ReferenceSchema):
    """Marshmallow schema for :class:`commercetools.types.CustomerGroupReference`."""

    obj = helpers.LazyNestedField(
        nested="commercetools._schemas._customer_group.CustomerGroupSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["type_id"]
        return types.CustomerGroupReference(**data)


class CustomerGroupResourceIdentifierSchema(ResourceIdentifierSchema):
    """Marshmallow schema for :class:`commercetools.types.CustomerGroupResourceIdentifier`."""

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["type_id"]
        return types.CustomerGroupResourceIdentifier(**data)


class CustomerGroupSchema(BaseResourceSchema):
    """Marshmallow schema for :class:`commercetools.types.CustomerGroup`."""

    id = marshmallow.fields.String(allow_none=True)
    version = marshmallow.fields.Integer(allow_none=True)
    created_at = marshmallow.fields.DateTime(allow_none=True, data_key="createdAt")
    last_modified_at = marshmallow.fields.DateTime(
        allow_none=True, data_key="lastModifiedAt"
    )
    last_modified_by = helpers.LazyNestedField(
        nested="commercetools._schemas._common.LastModifiedBySchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
        data_key="lastModifiedBy",
    )
    created_by = helpers.LazyNestedField(
        nested="commercetools._schemas._common.CreatedBySchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
        data_key="createdBy",
    )
    key = marshmallow.fields.String(allow_none=True, missing=None)
    name = marshmallow.fields.String(allow_none=True)
    custom = helpers.LazyNestedField(
        nested="commercetools._schemas._type.CustomFieldsSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        return types.CustomerGroup(**data)


class CustomerGroupUpdateActionSchema(marshmallow.Schema):
    """Marshmallow schema for :class:`commercetools.types.CustomerGroupUpdateAction`."""

    action = marshmallow.fields.String(allow_none=True)

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.CustomerGroupUpdateAction(**data)


class CustomerGroupUpdateSchema(marshmallow.Schema):
    """Marshmallow schema for :class:`commercetools.types.CustomerGroupUpdate`."""

    version = marshmallow.fields.Integer(allow_none=True)
    actions = marshmallow.fields.List(
        helpers.Discriminator(
            discriminator_field=("action", "action"),
            discriminator_schemas={
                "changeName": "commercetools._schemas._customer_group.CustomerGroupChangeNameActionSchema",
                "setCustomField": "commercetools._schemas._customer_group.CustomerGroupSetCustomFieldActionSchema",
                "setCustomType": "commercetools._schemas._customer_group.CustomerGroupSetCustomTypeActionSchema",
                "setKey": "commercetools._schemas._customer_group.CustomerGroupSetKeyActionSchema",
            },
            unknown=marshmallow.EXCLUDE,
            allow_none=True,
        ),
        allow_none=True,
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        return types.CustomerGroupUpdate(**data)


class CustomerGroupChangeNameActionSchema(CustomerGroupUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.CustomerGroupChangeNameAction`."""

    name = marshmallow.fields.String(allow_none=True)

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.CustomerGroupChangeNameAction(**data)


class CustomerGroupSetCustomFieldActionSchema(CustomerGroupUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.CustomerGroupSetCustomFieldAction`."""

    name = marshmallow.fields.String(allow_none=True)
    value = marshmallow.fields.Raw(allow_none=True, missing=None)

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.CustomerGroupSetCustomFieldAction(**data)


class CustomerGroupSetCustomTypeActionSchema(CustomerGroupUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.CustomerGroupSetCustomTypeAction`."""

    type = helpers.LazyNestedField(
        nested="commercetools._schemas._type.TypeResourceIdentifierSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
    )
    fields = FieldContainerField(allow_none=True, missing=None)  # type: ignore

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.CustomerGroupSetCustomTypeAction(**data)


class CustomerGroupSetKeyActionSchema(CustomerGroupUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.CustomerGroupSetKeyAction`."""

    key = marshmallow.fields.String(allow_none=True, missing=None)

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.CustomerGroupSetKeyAction(**data)
